package com.nokia.dingtalk_api.auth;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.nokia.dingtalk_api.common.R;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.access.AccessDeniedHandler;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

/**
 * 为Spring Security提供了统一的JSON响应格式，当出现未认证或无权限访问时，向客户端返回具体的错误信息和HTTP状态码
 */
@Slf4j
@Configuration
@RequiredArgsConstructor
public class MyExceptionHandling implements AuthenticationEntryPoint, AccessDeniedHandler {
    private final ObjectMapper objectMapper;
    /**
     * 当用户尝试访问一个需要身份验证但尚未认证的资源时，此方法会被调用
     */
    @Override
    public void commence(HttpServletRequest request, HttpServletResponse response, AuthenticationException authException) throws IOException, ServletException {
        log.warn(authException.toString());
        String s = objectMapper.writeValueAsString(R.error().message("认证失败").code(401));
        response.setStatus(HttpStatus.UNAUTHORIZED.value());
        response.setCharacterEncoding(StandardCharsets.UTF_8.name());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.getOutputStream().write(s.getBytes());
        log.warn("响应内容: {}", s);
        log.warn("响应状态: {}", response.getStatus());
    }

    /**
     * 当用户已经通过身份验证但没有足够的权限访问请求的资源时，此方法会被调用
     */
    @Override
    public void handle(HttpServletRequest request, HttpServletResponse response, AccessDeniedException accessDeniedException) throws IOException, ServletException {
        log.warn(accessDeniedException.toString());
        String s = objectMapper.writeValueAsString(R.error().message("禁止访问").code(403));
        response.setStatus(HttpStatus.FORBIDDEN.value());
        response.setCharacterEncoding(StandardCharsets.UTF_8.name());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.getOutputStream().write(s.getBytes());
        log.warn("响应内容: {}", s);
        log.warn("响应状态: {}", response.getStatus());
    }
}
