package com.nokia.dingtalk_api.auth.token;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.TypeFactory;
import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.config.AppConfig;
import com.nokia.dingtalk_api.dao.IAppService;
import com.nokia.dingtalk_api.pojos.po.AppPo;
import com.nokia.dingtalk_api.util.AesUtil;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.context.RequestAttributeSecurityContextRepository;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.security.web.util.matcher.RequestMatcher;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * 自定义的token认证过滤器，用于处理基于token的认证请求。
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class TokenAuthenticationFilter extends OncePerRequestFilter {
    public static final String PATTERN = "/api/open/**";
    private static final RequestMatcher REQUEST_MATCHER = new AntPathRequestMatcher(PATTERN);
    private final RequestAttributeSecurityContextRepository requestAttributeSecurityContextRepository;
    private final ObjectMapper objectMapper;
    private final IAppService iAppService;
    private final AppConfig appConfig;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {
        if (!REQUEST_MATCHER.matches(request)) {
            filterChain.doFilter(request, response);
            return;
        }
        String traceId = request.getHeader("TRACE_ID");
        if (!StringUtils.hasText(traceId)) {
            handleException(request, response, "TRACE_ID不能为空");
            return;
        }
        MDC.put("traceId", traceId);
        String token = request.getHeader("TOKEN");
        String appId = request.getHeader("APP_ID");
        String timestamp = request.getHeader("TIMESTAMP");
        if (!StringUtils.hasText(token)) {
            handleException(request, response, "TOKEN不能为空");
            return;
        }
        if (!StringUtils.hasText(appId)) {
            handleException(request, response, "APP_ID不能为空");
            return;
        }
        if (!StringUtils.hasText(timestamp)) {
            handleException(request, response, "TIMESTAMP不能为空");
            return;
        }
        response.setHeader("APP_ID", appId);
        response.setHeader("TIMESTAMP", timestamp);
        response.setHeader("TRACE_ID", traceId);
        // 根据appId查询应用信息
        AppPo appPo = iAppService.getById(appId);
        if (appPo == null) {
            handleException(request, response, "APP_ID无效");
            return;
        }
        String appSecret = appPo.getAppSecret();
        try {
            // 解密token
            String decryptedStr = AesUtil.decrypt(token, appSecret);
            Map<String, String> m = objectMapper.readValue(decryptedStr,
                    TypeFactory.defaultInstance().constructMapType(Map.class, String.class, String.class));
            String APP_ID = m.get("APP_ID");
            String TIMESTAMP = m.get("TIMESTAMP");
            String TRACE_ID = m.get("TRACE_ID");
            if (!appId.equals(APP_ID) || !timestamp.equals(TIMESTAMP) || !traceId.equals(TRACE_ID)) {
                handleException(request, response, "TOKEN无效");
                return;
            }
            LocalDateTime localDateTime = LocalDateTime.parse(TIMESTAMP,
                    DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS"));
            //  检查token是否过期
            if (Math.abs(Duration.between(localDateTime, LocalDateTime.now()).toMinutes()) > appConfig.getAppTokenExpire()) {
                handleException(request, response, "TOKEN已过期");
                return;
            }
        } catch (Exception e) {
            log.error(e.toString(), e);
            handleException(request, response, "TOKEN无效");
            return;
        }
        // 创建认证成功的 token 对象
        AuthenticationToken t = new AuthenticationToken(appId, null, null);
        // 设置认证请求的详细信息
        t.setDetails(appPo.getAppName());
        SecurityContext context = SecurityContextHolder.createEmptyContext();
        context.setAuthentication(t);
        requestAttributeSecurityContextRepository.saveContext(context, request, response);
        filterChain.doFilter(request, response);
    }

    private void handleException(HttpServletRequest request, HttpServletResponse response, String message) throws IOException {
        String traceId = MDC.get("traceId");
        if (!StringUtils.hasText(traceId)) {
            traceId = AesUtil.generateTraceId();
            MDC.put("traceId", traceId);
        }
        log.warn("请求地址: {} {}", request.getRequestURL().toString(), request.getMethod());
        // 请求头参数
        Map<String, String> headers = new HashMap<>();
        Enumeration<String> headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String k	= headerNames.nextElement();
            String v = request.getHeader(k);
            headers.put(k, v);
        }
        log.warn("请求头参数: {}", objectMapper.writeValueAsString(headers));
        // 查询参数
        Map<String, String> parameters = new HashMap<>();
        Enumeration<String> parameterNames = request.getParameterNames();
        while (parameterNames.hasMoreElements()) {
            String k	= parameterNames.nextElement();
            String v = request.getParameter(k);
            parameters.put(k, v);
        }
        log.warn("查询参数: {}", objectMapper.writeValueAsString(parameters));
        String s = objectMapper.writeValueAsString(R.error().message(message).code(401));
        response.setStatus(HttpStatus.UNAUTHORIZED.value());
        response.setCharacterEncoding(StandardCharsets.UTF_8.name());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.getOutputStream().write(s.getBytes());
        log.warn("响应内容: {}", s);
        log.warn("响应状态: {}", response.getStatus());
    }
}
