package com.nokia.dingtalk_api.config.security;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.nokia.dingtalk_api.auth.MyExceptionHandling;
import com.nokia.dingtalk_api.auth.UserDetailServiceImpl;
import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.pojos.po.RequestLogPo;
import com.nokia.dingtalk_api.pojos.vo.cms.LoginVo;
import com.nokia.dingtalk_api.service.RequestLogService;
import com.nokia.dingtalk_api.util.AesUtil;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.AnonymousAuthenticationProvider;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.ProviderManager;
import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.session.SessionRegistry;
import org.springframework.security.core.session.SessionRegistryImpl;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.DelegatingPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.logout.LogoutSuccessHandler;
import org.springframework.security.web.authentication.session.ChangeSessionIdAuthenticationStrategy;
import org.springframework.security.web.authentication.session.CompositeSessionAuthenticationStrategy;
import org.springframework.security.web.authentication.session.RegisterSessionAuthenticationStrategy;
import org.springframework.security.web.authentication.session.SessionAuthenticationStrategy;
import org.springframework.security.web.context.DelegatingSecurityContextRepository;
import org.springframework.security.web.context.HttpSessionSecurityContextRepository;
import org.springframework.security.web.context.RequestAttributeSecurityContextRepository;
import org.springframework.security.web.context.SecurityContextRepository;
import org.springframework.security.web.session.HttpSessionEventPublisher;
import org.springframework.util.StringUtils;

import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;

@Slf4j
@Configuration
@RequiredArgsConstructor
public class SecurityBeanConfig {
    private static final String KEY = UUID.randomUUID().toString();
    private final MyExceptionHandling myExceptionHandling;
    private final ObjectMapper objectMapper;
    private final UserDetailServiceImpl userDetailService;
    private final RequestLogService requestLogService;

    /**
     * 将 SecurityContext 存储在 HTTP 会话中
     */
    @Bean
    public HttpSessionSecurityContextRepository httpSessionSecurityContextRepository() {
        return new HttpSessionSecurityContextRepository();
    }

    /**
     * 将 SecurityContext 存储在当前请求的属性中
     */
    @Bean
    public RequestAttributeSecurityContextRepository requestAttributeSecurityContextRepository() {
        return new RequestAttributeSecurityContextRepository();
    }

    /**
     * 配置密码编码
     */
    @Bean
    public PasswordEncoder passwordEncoder() {
        // 默认使用的密码编码算法标识符
        String idForEncode = "bcrypt";
        Map<String, PasswordEncoder> m = new HashMap<>();
        m.put("bcrypt", new BCryptPasswordEncoder());
        return new DelegatingPasswordEncoder(idForEncode, m);
    }

    /**
     * 配置用户名密码认证
     */
    @Bean
    public DaoAuthenticationProvider daoAuthenticationProvider() {
        DaoAuthenticationProvider provider = new DaoAuthenticationProvider();
        provider.setPasswordEncoder(passwordEncoder());
        provider.setUserDetailsService(userDetailService);
        provider.setForcePrincipalAsString(true);
        return provider;
    }

    /**
     * 配置匿名认证
     */
    @Bean
    public AnonymousAuthenticationProvider anonymousAuthenticationProvider() {
        return new AnonymousAuthenticationProvider(KEY);
    }

    /**
     * 配置认证方式
     */
    @Bean
    public AuthenticationManager authenticationManager() {
        List<AuthenticationProvider> l = new ArrayList<>();
        l.add(anonymousAuthenticationProvider());
        l.add(daoAuthenticationProvider());
        return new ProviderManager(l);
    }

    /**
     * 配置认证存储
     */
    @Bean
    public DelegatingSecurityContextRepository delegatingSecurityContextRepository() {
        List<SecurityContextRepository> l = new ArrayList<>();
        l.add(httpSessionSecurityContextRepository());
        l.add(requestAttributeSecurityContextRepository());
        return new DelegatingSecurityContextRepository(l);
    }

    /**
     * 配置session监听
     */
    @Bean
    public HttpSessionEventPublisher httpSessionEventPublisher() {
        return new HttpSessionEventPublisher();
    }

    /**
     * 配置session策略
     */
    @Bean
    public CompositeSessionAuthenticationStrategy compositeSessionStrategy() {
        List<SessionAuthenticationStrategy> l = new ArrayList<>();
        l.add(new RegisterSessionAuthenticationStrategy(sessionRegistry()));
        l.add(new ChangeSessionIdAuthenticationStrategy());
        return new CompositeSessionAuthenticationStrategy(l);
    }

    /**
     * 跟踪和管理用户会话
     */
    @Bean
    public SessionRegistry sessionRegistry() {
        return new SessionRegistryImpl();
    }

    /**
     * 处理用户成功登出后的逻辑
     */
    @Bean
    public LogoutSuccessHandler logoutSuccessHandler() {
        return (request, response, authentication) -> {
            String traceId = getTraceId();
            log.debug("{} logout", authentication.getName());
            String s = objectMapper.writeValueAsString(R.ok().message("退出成功"));
            response.setCharacterEncoding(StandardCharsets.UTF_8.name());
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            response.getOutputStream().write(s.getBytes());
            saveRequestLog(request, authentication, traceId);
        };
    }

    /**
     * 用于处理用户成功登录后的逻辑
     */
    @Bean
    public AuthenticationSuccessHandler authenticationSuccessHandler() {
        return (request, response, authentication) -> {
            String traceId = getTraceId();
            log.debug("{} login", authentication.getName());
            LoginVo vo = new LoginVo();
            vo.setName(authentication.getName());
            String s = objectMapper.writeValueAsString(R.ok(vo).message("登录成功"));
            response.setCharacterEncoding(StandardCharsets.UTF_8.name());
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            response.getOutputStream().write(s.getBytes());
            saveRequestLog(request, authentication, traceId);
        };
    }

    /**
     * 处理用户登录失败后的逻辑
     */
    @Bean
    public AuthenticationFailureHandler authenticationFailureHandler() {
        return myExceptionHandling::commence;
    }

    /**
     * 保存请求日志
     *
     * @param request        请求
     * @param authentication 认证信息
     * @param traceId 请求跟踪id
     */
    private void saveRequestLog(HttpServletRequest request, Authentication authentication, String traceId)
            throws JsonProcessingException {
        LocalDateTime now = LocalDateTime.now();
        Map<String, String> headers = new HashMap<>();
        Enumeration<String> headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String k	= headerNames.nextElement();
            String v = request.getHeader(k);
            headers.put(k, v);
        }
        String ip = Optional.ofNullable(request.getHeader("X-Real-IP"))
                .or(() -> Optional.ofNullable(request.getHeader("X-Forwarded-For")))
                .orElse(request.getRemoteAddr());
        String principal = (String) authentication.getPrincipal();
        RequestLogPo requestLogPo = new RequestLogPo();
        requestLogPo.setRequestTime(now);
        requestLogPo.setApi(request.getRequestURI());
        requestLogPo.setHeaders(objectMapper.writeValueAsString(headers));
        requestLogPo.setIp(ip);
        requestLogPo.setApi(request.getRequestURI());
        requestLogPo.setOpen(0);
        requestLogPo.setUsername(principal);
        requestLogPo.setTraceId(traceId);
        requestLogService.offer(requestLogPo);
    }

    private static String getTraceId() {
        String traceId = MDC.get("traceId");
        if (!StringUtils.hasText(traceId)) {
            traceId = AesUtil.generateTraceId();
            MDC.put("traceId", traceId);
        }
        return traceId;
    }
}
