package com.nokia.dingtalk_api.config.security;

import com.nokia.dingtalk_api.auth.MyExceptionHandling;
import com.nokia.dingtalk_api.auth.token.TokenAuthenticationFilter;
import com.nokia.dingtalk_api.filter.SessionRefreshFilter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.security.servlet.PathRequest;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.access.intercept.AuthorizationFilter;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.security.web.authentication.logout.LogoutSuccessHandler;
import org.springframework.security.web.authentication.session.CompositeSessionAuthenticationStrategy;
import org.springframework.security.web.context.DelegatingSecurityContextRepository;
import org.springframework.security.web.context.RequestAttributeSecurityContextRepository;

@Slf4j
@RequiredArgsConstructor
@EnableMethodSecurity
@EnableWebSecurity
//@EnableWebSecurity(debug = true)
@Configuration
public class SecurityConfig {
    private final AuthenticationManager authenticationManager;
    private final MyExceptionHandling myExceptionHandling;
    private final RequestAttributeSecurityContextRepository requestAttributeSecurityContextRepository;
    private final DelegatingSecurityContextRepository delegatingSecurityContextRepository;
    private final CompositeSessionAuthenticationStrategy compositeSessionStrategy;
    private final LogoutSuccessHandler logoutSuccessHandler;
    private final AuthenticationSuccessHandler authenticationSuccessHandler;
    private final AuthenticationFailureHandler authenticationFailureHandler;
    private final SessionRefreshFilter sessionRefreshFilter;
    private final TokenAuthenticationFilter tokenAuthenticationFilter;

    @Bean
    @Order(1)
    public SecurityFilterChain tokenSecurityFilterChain(HttpSecurity httpSecurity) throws Exception {
        httpSecurity
                .securityMatcher(TokenAuthenticationFilter.PATTERN)
                .authorizeHttpRequests(t -> t.anyRequest().authenticated())
                .addFilterBefore(tokenAuthenticationFilter, UsernamePasswordAuthenticationFilter.class)
                .exceptionHandling(t -> t.authenticationEntryPoint(myExceptionHandling)
                        .accessDeniedHandler(myExceptionHandling))
                .securityContext(t -> t.securityContextRepository(requestAttributeSecurityContextRepository))
                .sessionManagement(AbstractHttpConfigurer::disable)
                .csrf(AbstractHttpConfigurer::disable)
                .formLogin(AbstractHttpConfigurer::disable)
                .logout(AbstractHttpConfigurer::disable)
                .rememberMe(AbstractHttpConfigurer::disable)
                .anonymous(AbstractHttpConfigurer::disable)
                .httpBasic(AbstractHttpConfigurer::disable);
        return httpSecurity.build();
    }

    @Bean
    public SecurityFilterChain sessionSecurityFilterChain(HttpSecurity httpSecurity) throws Exception {
        String loginUrl = "/api/cms/login";
        httpSecurity
                .authorizeHttpRequests(
                        t -> t.requestMatchers(PathRequest.toStaticResources().atCommonLocations()).permitAll()
                                .requestMatchers("/error", "/api/public/**", "/v3/**", "/doc.html", "/swagger-ui/**", loginUrl)
                                .permitAll()
                                .anyRequest().authenticated())
                .addFilterAfter(sessionRefreshFilter, AuthorizationFilter.class)
                .exceptionHandling(t -> t.authenticationEntryPoint(myExceptionHandling)
                        .accessDeniedHandler(myExceptionHandling))
                .formLogin(t -> t.successHandler(authenticationSuccessHandler)
                        .failureHandler(authenticationFailureHandler)
                        .loginProcessingUrl(loginUrl))
                .logout(t -> t.logoutSuccessHandler(logoutSuccessHandler).logoutUrl("/api/cms/logout"))
                .securityContext(t -> t.securityContextRepository(delegatingSecurityContextRepository))
                .sessionManagement(t -> t.sessionAuthenticationStrategy(compositeSessionStrategy))
                .authenticationManager(authenticationManager)
                .csrf(AbstractHttpConfigurer::disable)
                .httpBasic(AbstractHttpConfigurer::disable);
        return httpSecurity.build();
    }
}
