package com.nokia.dingtalk_api.config.web;

import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.common.exception.BizException;
import jakarta.validation.ValidationException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.TypeMismatchException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageConversionException;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MissingRequestValueException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;


/**
 * 请求异常处理
 */
@Slf4j
@RestControllerAdvice
public class ControllerExceptionHandler
{
    @ExceptionHandler(HttpRequestMethodNotSupportedException.class)
    public ResponseEntity<Object> httpRequestMethodNotSupportedExceptionHandler(HttpRequestMethodNotSupportedException e)
    {
        log.warn(e.getMessage());
        return ResponseEntity.status(HttpStatus.METHOD_NOT_ALLOWED).body(R.error().message(e.getMessage()));
    }

    @ExceptionHandler(MissingRequestValueException.class)
    public ResponseEntity<Object> missingRequestValueExceptionHandler(MissingRequestValueException e)
    {
        log.warn(e.getMessage());
        return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(R.error().message(e.getMessage()));
    }

    @ExceptionHandler(HttpMessageConversionException.class)
    public ResponseEntity<Object> httpMessageConversionExceptionHandler(HttpMessageConversionException e)
    {
        log.warn(e.getMessage());
        return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(R.error().message(e.getMessage()));
    }

    @ExceptionHandler(TypeMismatchException.class)
    public ResponseEntity<Object> typeMismatchExceptionHandler(TypeMismatchException e)
    {
        log.warn(e.getMessage());
        return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(R.error().message(e.getMessage()));
    }

    @ExceptionHandler(BindException.class)
    public ResponseEntity<Object> bindExceptionHandler(BindException e)
    {
        FieldError fieldError = e.getBindingResult().getFieldError();
        String message = "";
        if (fieldError != null)
        {
            message = fieldError.getDefaultMessage();
        }
        log.warn(message);
        return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(R.error().message(message));
    }

    @ExceptionHandler(ValidationException.class)
    public ResponseEntity<Object> validationExceptionHandler(ValidationException e)
    {
        log.warn(e.getMessage());
        return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(R.error().message(e.getMessage()));
    }

    @ExceptionHandler(AuthenticationException.class)
    public ResponseEntity<Object> authenticationExceptionHandler(AuthenticationException e)
    {
        log.warn(e.getMessage());
        return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(R.error().message("认证失败").code(401));
    }

    @ExceptionHandler(AccessDeniedException.class)
    public ResponseEntity<Object> accessDeniedExceptionHandler(AccessDeniedException e)
    {
        log.warn(e.getMessage());
        return ResponseEntity.status(HttpStatus.FORBIDDEN).body(R.error().message("禁止访问").code(403));
    }

    @ExceptionHandler(BizException.class)
    public ResponseEntity<Object> bizExceptionHandler(BizException e)
    {
        log.warn(e.getMessage());
        return ResponseEntity.ok().body(R.error(e.getMessage()));
    }

    @ExceptionHandler(Exception.class)
    public ResponseEntity<Object> exceptionHandler(Exception e)
    {
        log.error("╭( ′• o •′ )╭☞ 发生错误了 {}", e.toString(), e);
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(R.error());
    }
}
