package com.nokia.dingtalk_api.config.web;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.TypeFactory;
import com.nokia.dingtalk_api.pojos.po.RequestLogPo;
import com.nokia.dingtalk_api.service.RequestLogService;
import com.nokia.dingtalk_api.util.AesUtil;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.lang.Nullable;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.util.StopWatch;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import java.io.IOException;
import java.lang.reflect.Type;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 请求日志拦截器
 */
@Slf4j
@ControllerAdvice
@RequiredArgsConstructor
public class RequestLogHandlerInterceptor implements HandlerInterceptor, RequestBodyAdvice, ResponseBodyAdvice<Object> {
    /**
     * 计时器线程变量
     */
    private static final ThreadLocal<StopWatch> STOP_WATCH_THREAD_LOCAL = new ThreadLocal<>();
    private static final ThreadLocal<RequestLogPo> REQUEST_LOG_PO_THREAD_LOCAL = new ThreadLocal<>();
    private final ObjectMapper objectMapper;
    private final RequestLogService requestLogService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        // 计时器放入线程变量
        STOP_WATCH_THREAD_LOCAL.set(stopWatch);
        String traceId = MDC.get("traceId");
        if (!StringUtils.hasText(traceId)) {
            traceId = AesUtil.generateTraceId();
            MDC.put("traceId", traceId);
        }
        LocalDateTime now = LocalDateTime.now();
        log.info("请求地址: {} {}", request.getRequestURL().toString(), request.getMethod());
        // 请求头参数
        Map<String, String> headers = new HashMap<>();
        Enumeration<String> headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String k	= headerNames.nextElement();
            String v = request.getHeader(k);
            headers.put(k, v);
        }
        log.info("请求头参数: {}", objectMapper.writeValueAsString(headers));
        // 查询参数
        Map<String, String> parameters = new HashMap<>();
        Enumeration<String> parameterNames = request.getParameterNames();
        while (parameterNames.hasMoreElements()) {
            String k	= parameterNames.nextElement();
            String v = request.getParameter(k);
            parameters.put(k, v);
        }
        log.info("查询参数: {}", objectMapper.writeValueAsString(parameters));
        String ip = Optional.ofNullable(request.getHeader("X-Real-IP"))
                .or(() -> Optional.ofNullable(request.getHeader("X-Forwarded-For")))
                .orElse(request.getRemoteAddr());
        String token = request.getHeader("TOKEN");
        String timestamp = request.getHeader("TIMESTAMP");
        LocalDateTime timeStamp = StringUtils.hasText(timestamp)
                ? LocalDateTime.parse(timestamp, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS"))
                : null;
        SecurityContext context = SecurityContextHolder.getContext();
        Authentication authentication = context.getAuthentication();
        String principal = (String) authentication.getPrincipal();
        log.debug("principal: {}", authentication.getPrincipal());
        Map<String, String> requestParameters = new HashMap<>();
        requestParameters.put("request_parameters", objectMapper.writeValueAsString(parameters));
        boolean open = StringUtils.startsWithIgnoreCase(request.getRequestURI(), "/api/open/");
        RequestLogPo requestLogPo = new RequestLogPo();
        REQUEST_LOG_PO_THREAD_LOCAL.set(requestLogPo);
        requestLogPo.setRequestTime(now);
        requestLogPo.setIp(ip);
        requestLogPo.setApi(request.getRequestURI());
        requestLogPo.setOpen(open ? 1 : 0);
        requestLogPo.setRequestParameters(objectMapper.writeValueAsString(requestParameters));
        requestLogPo.setHeaders(objectMapper.writeValueAsString(headers));
        if (open) {
            requestLogPo.setAppId(principal);
            requestLogPo.setAppName(authentication.getDetails().toString());
        } else {
            requestLogPo.setUsername(principal);
        }
        requestLogPo.setTimeStamp(timeStamp);
        requestLogPo.setToken(token);
        requestLogPo.setTraceId(traceId);
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler,
                                @Nullable Exception ex) throws Exception {
        log.info("响应状态: {}", response.getStatus());
        StopWatch stopWatch = STOP_WATCH_THREAD_LOCAL.get();
        stopWatch.stop();
        log.info("耗时: {} ms", stopWatch.getTotalTimeMillis());
        STOP_WATCH_THREAD_LOCAL.remove();
        RequestLogPo requestLogPo = REQUEST_LOG_PO_THREAD_LOCAL.get();
        if (requestLogPo != null) {
            requestLogService.offer(requestLogPo);
        }
        REQUEST_LOG_PO_THREAD_LOCAL.remove();
    }

    @Override
    public boolean supports(MethodParameter methodParameter, Type targetType,
                            Class<? extends HttpMessageConverter<?>> converterType) {
        return true;
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage inputMessage, MethodParameter parameter, Type targetType,
                                           Class<? extends HttpMessageConverter<?>> converterType) throws IOException {
        return inputMessage;
    }

    @Override
    public Object afterBodyRead(Object body, HttpInputMessage inputMessage, MethodParameter parameter, Type targetType,
                                Class<? extends HttpMessageConverter<?>> converterType) {
        try {
            if (STOP_WATCH_THREAD_LOCAL.get() != null) {
                String bodyStr = objectMapper.writeValueAsString(body);
                log.info("请求体参数: {}", bodyStr);
                RequestLogPo requestLogPo = REQUEST_LOG_PO_THREAD_LOCAL.get();
                if (requestLogPo != null) {
                    Map<String, Object> requestParameters = objectMapper.readValue(requestLogPo.getRequestParameters(),
                            TypeFactory.defaultInstance().constructMapType(Map.class, String.class, String.class));
                    requestParameters.put("request_body", body);
                    requestLogPo.setRequestParameters(objectMapper.writeValueAsString(requestParameters));
                }
            }
        } catch (Exception e) {
            log.error(e.toString(), e);
        }
        return body;
    }

    @Override
    public Object handleEmptyBody(Object body, HttpInputMessage inputMessage, MethodParameter parameter,
                                  Type targetType, Class<? extends HttpMessageConverter<?>> converterType) {
        return body;
    }

    @Override
    public boolean supports(MethodParameter returnType, Class<? extends HttpMessageConverter<?>> converterType) {
        return true;
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter returnType, MediaType selectedContentType,
                                  Class<? extends HttpMessageConverter<?>> selectedConverterType,
                                  ServerHttpRequest request, ServerHttpResponse response) {
        try {
            if (STOP_WATCH_THREAD_LOCAL.get() != null) {
                log.info("响应内容: {}",
                        org.apache.commons.lang3.StringUtils.substring(objectMapper.writeValueAsString(body),
                                0, 500));
            }
        } catch (Exception e) {
            log.error(e.toString(), e);
        }
        return body;
    }
}