package com.nokia.dingtalk_api.filter;

import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.Cookie;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.servlet.http.HttpSession;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;

@Slf4j
@Component
public class SessionRefreshFilter extends OncePerRequestFilter {
    /**
     * Cookie的名称
     */
    static final String COOKIE_NAME = "JSESSIONID";
    /**
     * 通过@Value注解注入Spring配置中的session超时时间
     */
    @Value("${spring.session.timeout}")
    private Integer sessionTimeout;
    /**
     * 通过@Value注解注入Spring配置中的cookie最大存活时间
     */
    @Value("${server.servlet.session.cookie.max-age}")
    private Integer cookieMaxAge;
    @Value("${server.servlet.session.cookie.http-only}")
    private Boolean httpOnly;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        // 从SecurityContextHolder中获取当前的Authentication对象
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        // 检查Authentication对象是否存在且已认证
        if (authentication != null && authentication.isAuthenticated()) {
            // 尝试从HttpServletRequest中获取HttpSession，如果不存在则返回null
            HttpSession session = request.getSession(false);
            // 确保session不为null
            if (session != null) {
                // 设置session的超时时间
                session.setMaxInactiveInterval(sessionTimeout);
                // 创建一个新的Cookie，名称为COOKIE_NAME，值为session的ID
                Cookie cookie = new Cookie(COOKIE_NAME, session.getId());
                // 设置Cookie的最大存活时间
                cookie.setMaxAge(cookieMaxAge);
                // 设置Cookie的路径，这里设置为根路径
                cookie.setPath("/");
                // 设置Cookie为HttpOnly，增加安全性，防止JavaScript读取
                cookie.setHttpOnly(httpOnly);
                // 设置Cookie为Secure，确保Cookie只通过HTTPS协议传输
//                cookie.setSecure(true);
                // 将更新后的Cookie添加到HTTP响应中
                response.addCookie(cookie);
                log.debug(">>>>>>>>>update session: {}", session.getId());
            }
        }
        // 调用过滤器链中的下一个过滤器
        filterChain.doFilter(request, response);
    }
}
