package com.nokia.dingtalk_api.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.common.exception.BizException;
import com.nokia.dingtalk_api.dao.IAppRobotService;
import com.nokia.dingtalk_api.dao.IAppTaskService;
import com.nokia.dingtalk_api.dao.IRobotCommandService;
import com.nokia.dingtalk_api.mapper.AppRobotMapper;
import com.nokia.dingtalk_api.pojos.dto.cms.AddAppRobotDto;
import com.nokia.dingtalk_api.pojos.dto.cms.DeleteAppRobotDto;
import com.nokia.dingtalk_api.pojos.dto.cms.GetAppRobotOptionsDto;
import com.nokia.dingtalk_api.pojos.dto.cms.ListAppRobotDto;
import com.nokia.dingtalk_api.pojos.dto.cms.ListAvailableRobotDto;
import com.nokia.dingtalk_api.pojos.enums.AppTaskStatusEnum;
import com.nokia.dingtalk_api.pojos.enums.SortEnum;
import com.nokia.dingtalk_api.pojos.po.AppRobotPo;
import com.nokia.dingtalk_api.pojos.po.AppTaskPo;
import com.nokia.dingtalk_api.pojos.po.RobotCommandPo;
import com.nokia.dingtalk_api.pojos.vo.PageVo;
import com.nokia.dingtalk_api.pojos.vo.cms.GetAppRobotOptionsVo;
import com.nokia.dingtalk_api.pojos.vo.cms.ListAppRobotVo;
import com.nokia.dingtalk_api.pojos.vo.cms.ListAvailableRobotVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class AppRobotService {
    private final AppRobotMapper appRobotMapper;
    private final IAppRobotService iAppRobotService;
    private final AesService aesService;
    private final IAppTaskService iAppTaskService;
    private final IRobotCommandService iRobotCommandService;

    public R<PageVo<ListAppRobotVo>> listAppRobot(ListAppRobotDto dto) {
        Page<ListAppRobotVo> page = new Page<>(dto.getCurrent(), dto.getPageSize());
        if (CollectionUtils.isEmpty(dto.getSort())) {
            page.addOrder(OrderItem.asc("robot_code"));
        } else {
            List<OrderItem> items = dto.getSort().entrySet().stream()
                    .map(t -> t.getValue().equals(SortEnum.ascend) ? OrderItem.asc(t.getKey().value)
                            : OrderItem.desc(t.getKey().value)).toList();
            page.addOrder(items);
        }
        List<ListAppRobotVo> vos = appRobotMapper.listAppRobot(page, dto);
        vos.forEach(vo -> vo.setRobotSecret(aesService.decrypt(vo.getRobotSecret())));
        return R.ok(new PageVo<>(page.getTotal(), vos));
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> addAppRobot(AddAppRobotDto dto) {
        List<AppRobotPo> list = dto.getRobotCodes().stream().map(t -> new AppRobotPo(dto.getAppId(), t)).toList();
        iAppRobotService.saveBatch(list);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> deleteAppRobot(DeleteAppRobotDto dto) {
        // 检查关联任务
        QueryWrapper<AppTaskPo> wrapper1 = new QueryWrapper<>();
        wrapper1.eq("app_id", dto.getAppId());
        wrapper1.ne("status", AppTaskStatusEnum.DELETED.value);
        wrapper1.in("robot_code", dto.getRobotCodes());
        boolean exists = iAppTaskService.exists(wrapper1);
        if (exists) {
            throw new BizException("机器人被任务使用，无法删除");
        }
        // 删除应用机器人
        UpdateWrapper<AppRobotPo> wrapper2 = new UpdateWrapper<>();
        wrapper2.eq("app_id", dto.getAppId());
        wrapper2.in("robot_code", dto.getRobotCodes());
        appRobotMapper.delete(wrapper2);
        // 删除应用机器人命令
        UpdateWrapper<RobotCommandPo> wrapper3 = new UpdateWrapper<>();
        wrapper3.eq("app_id", dto.getAppId());
        wrapper3.in("robot_code", dto.getRobotCodes());
        iRobotCommandService.remove(wrapper3);
        return R.ok();
    }

    public R<PageVo<ListAvailableRobotVo>> listAvailableRobot(ListAvailableRobotDto dto) {
        Page<ListAvailableRobotVo> page = new Page<>(dto.getCurrent(), dto.getPageSize());
        if (CollectionUtils.isEmpty(dto.getSort())) {
            page.addOrder(OrderItem.asc("robot_code"));
        } else {
            List<OrderItem> items = dto.getSort().entrySet().stream()
                    .map(t -> t.getValue().equals(SortEnum.ascend) ? OrderItem.asc(t.getKey().value)
                            : OrderItem.desc(t.getKey().value)).toList();
            page.addOrder(items);
        }
        List<ListAvailableRobotVo> vos = appRobotMapper.listAvailableRobot(page, dto);
        vos.forEach(vo -> vo.setRobotSecret(aesService.decrypt(vo.getRobotSecret())));
        return R.ok(new PageVo<>(page.getTotal(), vos));
    }

    public R<List<GetAppRobotOptionsVo>> getAppRobotOptions(GetAppRobotOptionsDto dto) {
        ListAppRobotDto listAppRobotDto = new ListAppRobotDto();
        listAppRobotDto.setAppId(dto.getAppId());
        Page<ListAppRobotVo> page = new Page<>(1, Integer.MAX_VALUE);
        page.addOrder(OrderItem.asc("robot_name"));
        List<ListAppRobotVo> list = appRobotMapper.listAppRobot(page, listAppRobotDto);
        List<GetAppRobotOptionsVo> vos = list.stream().map(GetAppRobotOptionsVo::new).toList();
        return R.ok(vos);
    }
}
