package com.nokia.dingtalk_api.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.dao.IAppService;
import com.nokia.dingtalk_api.dao.IAppSftpService;
import com.nokia.dingtalk_api.dao.IAppTaskService;
import com.nokia.dingtalk_api.dao.IRobotCommandService;
import com.nokia.dingtalk_api.mapper.AppRobotMapper;
import com.nokia.dingtalk_api.pojos.dto.cms.AddAppDto;
import com.nokia.dingtalk_api.pojos.dto.cms.DeleteAppDto;
import com.nokia.dingtalk_api.pojos.dto.cms.ListAppDto;
import com.nokia.dingtalk_api.pojos.dto.cms.ResetAppSecretDto;
import com.nokia.dingtalk_api.pojos.dto.cms.UpdateAppDto;
import com.nokia.dingtalk_api.pojos.enums.AppTaskStatusEnum;
import com.nokia.dingtalk_api.pojos.enums.SortEnum;
import com.nokia.dingtalk_api.pojos.po.AppPo;
import com.nokia.dingtalk_api.pojos.po.AppRobotPo;
import com.nokia.dingtalk_api.pojos.po.AppSftpPo;
import com.nokia.dingtalk_api.pojos.po.AppTaskPo;
import com.nokia.dingtalk_api.pojos.po.RobotCommandPo;
import com.nokia.dingtalk_api.pojos.vo.PageVo;
import com.nokia.dingtalk_api.pojos.vo.cms.ListAppVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class AppService {
    private final IAppService iAppService;
    private final AppRobotMapper appRobotMapper;
    private final IRobotCommandService iRobotCommandService;
    private final IAppSftpService iAppSftpService;
    private final IAppTaskService iAppTaskService;
    private final AppTaskService appTaskService;

    public R<PageVo<ListAppVo>> listApp(ListAppDto dto) {
        QueryWrapper<AppPo> wrapper = new QueryWrapper<>();
        wrapper.eq("deleted", 0);
        if (StringUtils.hasText(dto.getAppId())) {
            wrapper.eq("app_id", dto.getAppId());
        }
        if (StringUtils.hasText(dto.getAppName())) {
            wrapper.like("app_name", "%" + dto.getAppName() + "%");
        }
        Page<AppPo> page = new Page<>(dto.getCurrent(), dto.getPageSize());
        if (CollectionUtils.isEmpty(dto.getSort())) {
            page.addOrder(OrderItem.desc("update_time"));
        } else {
            List<OrderItem> items = dto.getSort().entrySet().stream()
                    .map(t -> t.getValue().equals(SortEnum.ascend) ? OrderItem.asc(t.getKey().value)
                            : OrderItem.desc(t.getKey().value)).toList();
            page.addOrder(items);
        }
        List<AppPo> list = iAppService.list(page, wrapper);
        List<ListAppVo> vos = list.stream().map(ListAppVo::new).toList();
        return R.ok(new PageVo<>(page.getTotal(), vos));
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> addApp(AddAppDto dto) {
        QueryWrapper<AppPo> wrapper1 = new QueryWrapper<>();
        wrapper1.eq("app_id", dto.getAppId());
        boolean appIdExists = iAppService.exists(wrapper1);
        if (appIdExists) {
            return R.error("应用编号已存在");
        }
        QueryWrapper<AppPo> wrapper2 = new QueryWrapper<>();
        wrapper2.eq("app_name", dto.getAppName());
        wrapper2.eq("deleted", 0);
        boolean appNameExists = iAppService.exists(wrapper2);
        if (appNameExists) {
            return R.error("应用名称已存在");
        }
        AppPo po = new AppPo(dto);
        iAppService.save(po);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> updateApp(UpdateAppDto dto) {
        QueryWrapper<AppPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_name", dto.getAppName());
        wrapper.eq("deleted", 0);
        wrapper.ne("app_id", dto.getAppId());
        boolean appNameExists = iAppService.exists(wrapper);
        if (appNameExists) {
            return R.error("应用名称已存在");
        }
        AppPo po = new AppPo(dto);
        iAppService.updateById(po);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> resetAppSecret(ResetAppSecretDto dto) {
        AppPo po = new AppPo(dto);
        iAppService.updateById(po);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> deleteApp(DeleteAppDto dto) {
        AppPo po = new AppPo(dto);
        iAppService.updateById(po);
        // 删除应用机器人
        UpdateWrapper<AppRobotPo> wrapper1 = new UpdateWrapper<>();
        wrapper1.eq("app_id", dto.getAppId());
        appRobotMapper.delete(wrapper1);
        // 删除应用机器人命令
        UpdateWrapper<RobotCommandPo> wrapper2 = new UpdateWrapper<>();
        wrapper2.eq("app_id", dto.getAppId());
        iRobotCommandService.remove(wrapper2);
        // 删除应用sftp
        UpdateWrapper<AppSftpPo> wrapper3 = new UpdateWrapper<>();
        wrapper2.eq("app_id", dto.getAppId());
        iAppSftpService.remove(wrapper3);
        // 删除应用定时任务
        UpdateWrapper<AppTaskPo> wrapper4 = new UpdateWrapper<>();
        wrapper4.eq("app_id", dto.getAppId());
        List<AppTaskPo> appTaskPos = iAppTaskService.list(wrapper4);
        AppTaskPo appTaskPo = new AppTaskPo();
        appTaskPo.setStatus(AppTaskStatusEnum.DELETED.value);
        iAppTaskService.update(appTaskPo, wrapper4);
        appTaskPos.forEach(t -> appTaskService.removeTask(t.getTaskId()));
        return R.ok();
    }
}
