package com.nokia.dingtalk_api.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.common.exception.BizException;
import com.nokia.dingtalk_api.dao.IAppSftpService;
import com.nokia.dingtalk_api.dao.IAppTaskService;
import com.nokia.dingtalk_api.pojos.dto.cms.AddAppSftpDto;
import com.nokia.dingtalk_api.pojos.dto.cms.DeleteAppSftpDto;
import com.nokia.dingtalk_api.pojos.dto.cms.GetAppSftpOptionsDto;
import com.nokia.dingtalk_api.pojos.dto.cms.ListAppSftpDto;
import com.nokia.dingtalk_api.pojos.dto.cms.UpdateAppSftpDto;
import com.nokia.dingtalk_api.pojos.enums.AppTaskStatusEnum;
import com.nokia.dingtalk_api.pojos.enums.SortEnum;
import com.nokia.dingtalk_api.pojos.po.AppSftpPo;
import com.nokia.dingtalk_api.pojos.po.AppTaskPo;
import com.nokia.dingtalk_api.pojos.vo.PageVo;
import com.nokia.dingtalk_api.pojos.vo.cms.GetAppSftpOptionsVo;
import com.nokia.dingtalk_api.pojos.vo.cms.ListAppSftpVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.sshd.sftp.client.SftpClient;
import org.springframework.integration.file.remote.SessionCallbackWithoutResult;
import org.springframework.integration.file.remote.session.Session;
import org.springframework.integration.sftp.session.DefaultSftpSessionFactory;
import org.springframework.integration.sftp.session.SftpRemoteFileTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class AppSftpService {
    private final IAppSftpService iAppSftpService;
    private final AesService aesService;
    private final IAppTaskService iAppTaskService;

    /**
     * 检查sftp
     * @param host 主机
     * @param port 端口
     * @param username 用户
     * @param password 密码
     */
    public void checkSftp(String host, Integer port, String username, String password) {
        DefaultSftpSessionFactory factory = new DefaultSftpSessionFactory();
        factory.setHost(host);
        factory.setPort(port);
        factory.setUser(username);
        factory.setPassword(password);
        factory.setAllowUnknownKeys(true);
        SftpRemoteFileTemplate template = new SftpRemoteFileTemplate(factory);
        try {
            template.execute((SessionCallbackWithoutResult<SftpClient.DirEntry>) Session::test);
        } catch (Exception e) {
            log.error(e.toString(), e);
            throw new BizException("sftp 连接异常");
        }
    }

    public R<PageVo<ListAppSftpVo>> listAppSftp(ListAppSftpDto dto) {
        QueryWrapper<AppSftpPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", dto.getAppId());
        if (StringUtils.hasText(dto.getSftpId())) {
            wrapper.eq("sftp_id", dto.getSftpId());
        }
        if (StringUtils.hasText(dto.getHost())) {
            wrapper.like("host", "%" + dto.getHost() + "%");
        }
        if (dto.getPort() != null) {
            wrapper.eq("port", dto.getPort());
        }
        if (StringUtils.hasText(dto.getUsername())) {
            wrapper.like("user", "%" + dto.getUsername() + "%");
        }
        Page<AppSftpPo> page = new Page<>(dto.getCurrent(), dto.getPageSize());
        if (CollectionUtils.isEmpty(dto.getSort())) {
            page.addOrder(OrderItem.asc("sftp_id"));
        } else {
            List<OrderItem> items = dto.getSort().entrySet().stream()
                    .map(t -> t.getValue().equals(SortEnum.ascend) ? OrderItem.asc(t.getKey().value)
                            : OrderItem.desc(t.getKey().value)).toList();
            page.addOrder(items);
        }
        List<AppSftpPo> list = iAppSftpService.list(page, wrapper);
        List<ListAppSftpVo> vos = list.stream().map(ListAppSftpVo::new).toList();
        vos.forEach(vo -> vo.setPassword(aesService.decrypt(vo.getPassword())));
        return R.ok(new PageVo<>(page.getTotal(), vos));
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> addAppSftp(AddAppSftpDto dto) {
        QueryWrapper<AppSftpPo> wrapper = new QueryWrapper<>();
        wrapper.eq("sftp_id", dto.getSftpId());
        boolean exists = iAppSftpService.exists(wrapper);
        if (exists) {
            return R.error("sftp id已存在");
        }
        checkSftp(dto.getHost(), dto.getPort(), dto.getUsername(), dto.getPassword());
        dto.setPassword(aesService.encrypt(dto.getPassword()));
        AppSftpPo po = new AppSftpPo(dto);
        iAppSftpService.save(po);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> updateAppSftp(UpdateAppSftpDto dto) {
        checkSftp(dto.getHost(), dto.getPort(), dto.getUsername(), dto.getPassword());
        dto.setPassword(aesService.encrypt(dto.getPassword()));
        AppSftpPo po = new AppSftpPo(dto);
        iAppSftpService.updateById(po);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> deleteAppSftp(DeleteAppSftpDto dto) {
        // 检查关联任务
        QueryWrapper<AppTaskPo> wrapper = new QueryWrapper<>();
        wrapper.eq("sftp_id", dto.getSftpId());
        wrapper.ne("status", AppTaskStatusEnum.DELETED.value);
        boolean exists = iAppTaskService.exists(wrapper);
        if (exists) {
            throw new BizException("sftp被任务使用，无法删除");
        }
        iAppSftpService.removeById(dto.getSftpId());
        return R.ok();
    }

    public R<List<GetAppSftpOptionsVo>> getAppSftpOptions(GetAppSftpOptionsDto dto) {
        QueryWrapper<AppSftpPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", dto.getAppId());
        Page<AppSftpPo> page = new Page<>(1, Integer.MAX_VALUE);
        page.addOrder(OrderItem.asc("sftp_id"));
        List<AppSftpPo> list = iAppSftpService.list(page, wrapper);
        List<GetAppSftpOptionsVo> vos = list.stream().map(GetAppSftpOptionsVo::new).toList();
        return R.ok(vos);
    }
}
