package com.nokia.dingtalk_api.service;

import com.aliyun.dingtalkrobot_1_0.models.BatchSendOTOResponse;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupSendResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dingtalk.api.response.OapiMediaUploadResponse;
import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.common.exception.BizException;
import com.nokia.dingtalk_api.common.exception.MyRuntimeException;
import com.nokia.dingtalk_api.dao.IAppTaskLogService;
import com.nokia.dingtalk_api.dao.IAppTaskService;
import com.nokia.dingtalk_api.dao.IRetryAppTaskService;
import com.nokia.dingtalk_api.dao.ISendMessageLogService;
import com.nokia.dingtalk_api.mapper.AlertConfigMapper;
import com.nokia.dingtalk_api.mapper.AppTaskMapper;
import com.nokia.dingtalk_api.mapper.RetryAppTaskMapper;
import com.nokia.dingtalk_api.pojos.bo.AppTaskBo;
import com.nokia.dingtalk_api.pojos.dto.cms.AddAppTaskDto;
import com.nokia.dingtalk_api.pojos.dto.cms.DeleteAppTaskDto;
import com.nokia.dingtalk_api.pojos.dto.cms.EnableAppTaskDto;
import com.nokia.dingtalk_api.pojos.dto.cms.ListAppTaskDto;
import com.nokia.dingtalk_api.pojos.dto.cms.ListAppTaskLogDto;
import com.nokia.dingtalk_api.pojos.dto.cms.RunAppTaskDto;
import com.nokia.dingtalk_api.pojos.dto.cms.StopAppTaskDto;
import com.nokia.dingtalk_api.pojos.dto.cms.UpdateAppTaskDto;
import com.nokia.dingtalk_api.pojos.enums.AppTaskStatusEnum;
import com.nokia.dingtalk_api.pojos.enums.FileMethodEnum;
import com.nokia.dingtalk_api.pojos.enums.SortEnum;
import com.nokia.dingtalk_api.pojos.enums.SubdirectoryMethodEnum;
import com.nokia.dingtalk_api.pojos.enums.TaskAlertTypeEnum;
import com.nokia.dingtalk_api.pojos.po.AppTaskLogPo;
import com.nokia.dingtalk_api.pojos.po.AppTaskPo;
import com.nokia.dingtalk_api.pojos.po.RetryAppTaskPo;
import com.nokia.dingtalk_api.pojos.po.SendMessageLogPo;
import com.nokia.dingtalk_api.pojos.vo.PageVo;
import com.nokia.dingtalk_api.pojos.vo.cms.GetAlertConfigVo;
import com.nokia.dingtalk_api.pojos.vo.cms.ListAppTaskVo;
import com.nokia.dingtalk_api.util.DingTalkApiUtil;
import com.taobao.api.FileItem;
import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.sshd.sftp.client.SftpClient;
import org.slf4j.MDC;
import org.springframework.integration.file.remote.SessionCallbackWithoutResult;
import org.springframework.integration.sftp.session.DefaultSftpSessionFactory;
import org.springframework.integration.sftp.session.SftpRemoteFileTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.Instant;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class AppTaskService {
    /**
     * 英文逗号分割的手机号列表
     */
    static final Pattern PHONE_LIST_PATTERN = Pattern.compile("\\d{11}(,\\d{11})*");
    /**
     * 英文逗号分割的userId列表
     */
    static final Pattern USER_ID_LIST_PATTERN = Pattern.compile("\\w+(,\\w+)*");
    /**
     * 运行中的定时任务
     */
    static final ConcurrentMap<String, ScheduledFuture<?>> TASKS = new ConcurrentHashMap<>();
    /**
     * 重试定时任务
     */
    static final ConcurrentMap<String, ScheduledFuture<?>> RETRY_TASKS = new ConcurrentHashMap<>();
    private final ThreadPoolTaskScheduler taskScheduler;
    private final AppTaskMapper appTaskMapper;
    private final IAppTaskLogService iAppTaskLogService;
    private final OpenService openService;
    private final AesService aesService;
    private final IAppTaskService iAppTaskService;
    private final IRetryAppTaskService iRetryAppTaskService;
    private final RetryAppTaskMapper retryAppTaskMapper;
    private final AlertConfigMapper alertConfigMapper;
    private final ISendMessageLogService iSendMessageLogService;

    /**
     * 应用定时任务初始化
     */
    @PostConstruct
    public void init() {
        List<AppTaskBo> l1 = appTaskMapper.getRunningAppTasks(AppTaskStatusEnum.RUNNING.value);
        l1.forEach(this::addTask);
        List<AppTaskBo> l2 = retryAppTaskMapper.getRetryAppTasks(AppTaskStatusEnum.RUNNING.value);
        l2.forEach(t -> {
            ScheduledFuture<?> future = taskScheduler.schedule(() -> runTask(t), t.getRetryTime());
            log.debug("getActiveCount: {}", taskScheduler.getActiveCount());
            RETRY_TASKS.put(t.getTaskId(), future);
        });
    }

    /**
     * 删除任务
     * @param id 任务id
     */
    public void removeTask(String id) {
        // 删除定时任务
        ScheduledFuture<?> f1 = TASKS.get(id);
        if (f1 != null && !f1.isCancelled()) {
            f1.cancel(false);
        }
        TASKS.remove(id);
        // 删除重试任务
        ScheduledFuture<?> f2 = RETRY_TASKS.get(id);
        if (f2 != null && !f2.isCancelled()) {
            f2.cancel(false);
        }
        iRetryAppTaskService.removeById(id);
        RETRY_TASKS.remove(id);
    }

    /**
     * 添加任务
     * @param t 任务
     */
    public void addTask(AppTaskBo t) {
        removeTask(t.getTaskId());
        CronTrigger cronTrigger = new CronTrigger(t.getCron());
        ScheduledFuture<?> future = taskScheduler.schedule(() -> runTask(t), cronTrigger);
        log.debug("getActiveCount: {}", taskScheduler.getActiveCount());
        TASKS.put(t.getTaskId(), future);
    }

    /**
     * 运行任务
     * @param t 任务
     */
    public void runTask(AppTaskBo t) {
        AppTaskLogPo appTaskLogPo = new AppTaskLogPo(t);
        MDC.put("traceId", t.getTaskId());
        Set<String> fileList = new HashSet<>();
        Set<String> processQueryKeys = new HashSet<>();
        try {
            Runnable runnable = () -> {
                MDC.put("traceId", t.getTaskId());
                log.info("开始执行应用任务: {}", t);
                DefaultSftpSessionFactory factory = new DefaultSftpSessionFactory();
                factory.setHost(t.getHost());
                factory.setPort(t.getPort());
                factory.setUser(t.getUsername());
                factory.setPassword(aesService.decrypt(t.getPassword()));
                factory.setAllowUnknownKeys(true);
                SftpRemoteFileTemplate template = new SftpRemoteFileTemplate(factory);
                // 没有子文件夹
                if (t.getHasSubdirectory().equals(0)) {
                    findFile(t, template, t.getMasterDir(), fileList, processQueryKeys);
                } else {
                    if (!template.exists(t.getMasterDir())) {
                        throw new BizException("远端服务器找不到路径" + t.getMasterDir());
                    }
                    SftpClient.DirEntry[] dirEntries = template.list(t.getMasterDir());
                    log.debug("dirEntries: {}", Arrays.stream(dirEntries)
                            .map(SftpClient.DirEntry::getFilename).sorted().toList());
                    // 正则表达式匹配最新子文件夹
                    if (SubdirectoryMethodEnum.REG_EXP.name().equals(t.getSubdirectoryMethod())) {
                        Pattern pattern = Pattern.compile(t.getSubdirectoryPattern());
                        Optional<SftpClient.DirEntry> dirEntryOptional = Arrays.stream(dirEntries)
                                .filter(tt -> !".".equals(tt.getFilename()) && !"..".equals(tt.getFilename())
                                        && tt.getAttributes().isDirectory() && pattern.matcher(tt.getFilename()).find())
                                .max(Comparator.comparing(tt -> tt.getAttributes().getModifyTime()));
                        SftpClient.DirEntry dirEntry = dirEntryOptional.orElseThrow(() ->
                                new BizException("远端服务器没有找到匹配的子文件夹"));
                        String dirPath = t.getMasterDir() + (t.getMasterDir().endsWith("/") ? "" : "/")
                                + dirEntry.getFilename();
                        findFile(t, template, dirPath, fileList, processQueryKeys);
                    } else if (SubdirectoryMethodEnum.MONTH.name().equals(t.getSubdirectoryMethod())) {
                        // 按月匹配文件夹
                        LocalDate localDate;
                        if (StringUtils.hasText(t.getDateString())) {
                            localDate = dateStringParse(t);
                        } else {
                            localDate = LocalDate.now().plusMonths(t.getDirTimeDelay());
                        }
                        String month = localDate.format(DateTimeFormatter.ofPattern("yyyyMM"));
                        String dirPath = t.getMasterDir() + (t.getMasterDir().endsWith("/") ? "" : "/") + month;
                        findFile(t, template, dirPath, fileList, processQueryKeys);
                    } else if (SubdirectoryMethodEnum.DAY.name().equals(t.getSubdirectoryMethod())) {
                        // 按天匹配文件夹
                        LocalDate localDate;
                        if (StringUtils.hasText(t.getDateString())) {
                            localDate = dateStringParse(t);
                        } else {
                            localDate = LocalDate.now().plusDays(t.getDirTimeDelay());
                        }
                        String day = localDate.format(DateTimeFormatter.ofPattern("yyyyMMdd"));
                        String dirPath = t.getMasterDir() + (t.getMasterDir().endsWith("/") ? "" : "/") + day;
                        findFile(t, template, dirPath, fileList, processQueryKeys);
                    }
                }

                // 执行完删除重试任务
                iRetryAppTaskService.removeById(t.getTaskId());
                RETRY_TASKS.remove(t.getTaskId());
                log.info("结束应用任务: {}", t);
                String content = t.getAppName() + " " + t.getTaskName() + " 执行成功，消息id：" + processQueryKeys;
                sendMessage(t, content);
            };
            if (t.getTaskTimeout() > 0) {
                CompletableFuture.runAsync(runnable, taskScheduler).get(t.getTaskTimeout(), TimeUnit.SECONDS);
                log.debug("getActiveCount: {}", taskScheduler.getActiveCount());
            } else {
                CompletableFuture.runAsync(runnable, taskScheduler).join();
                log.debug("getActiveCount: {}", taskScheduler.getActiveCount());
            }
        } catch (InterruptedException e) {
            appTaskLogPo.setStatus(0);
            appTaskLogPo.setDetail(e.toString());
            log.error("线程中断: {}, {}", t, e, e);
            Thread.currentThread().interrupt();
            alert(t, "任务意外中断");
            retry(t);
        } catch (TimeoutException e) {
            appTaskLogPo.setStatus(0);
            appTaskLogPo.setDetail(e.toString());
            log.error("应用任务执行超时: {}, {}", t, e, e);
            alert(t, "任务执行超时");
            retry(t);
        } catch (Exception e) {
            appTaskLogPo.setStatus(0);
            appTaskLogPo.setDetail(e.toString());
            log.error("应用任务执行失败: {}, {}", t, e, e);
            Throwable rootCause = e;
            while (rootCause.getCause() != null && rootCause.getCause() != rootCause) {
                rootCause = rootCause.getCause();
            }
            alert(t, rootCause.getMessage());
            retry(t);
        } finally {
            // 记录日志
            appTaskLogPo.setFiles(fileList.toString());
            appTaskLogPo.setProcessQueryKeys(processQueryKeys.toString());
            iAppTaskLogService.save(appTaskLogPo);
        }
    }

    public static LocalDate dateStringParse(AppTaskBo t) {
        LocalDate localDate;
        try {
            localDate = LocalDate.parse(t.getDateString(), DateTimeFormatter.ofPattern("yyyyMMdd"));
        } catch (Exception e) {
            throw new BizException("日期参数格式错误");
        }
        return localDate;
    }

    /**
     * 重试
     * @param t 任务信息
     */
    public void retry(AppTaskBo t) {
        try {
            log.debug("retryTimes: {} < {} {}", t.getRetryTimes(), t.getMaxRetryTimes(), t.getRetryTimes() < t.getMaxRetryTimes());
            if (t.getRetryTimes() < t.getMaxRetryTimes()) {
                Instant instant = Instant.now().plusSeconds(t.getRetryInterval());
                RetryAppTaskPo retryAppTaskPo = new RetryAppTaskPo();
                retryAppTaskPo.setRetryTime(instant);
                retryAppTaskPo.setTaskId(t.getTaskId());
                retryAppTaskPo.setRetryTimes(t.getRetryTimes() + 1);
                iRetryAppTaskService.saveOrUpdate(retryAppTaskPo);
                AppTaskBo n = new AppTaskBo(t);
                n.setRetryTimes(n.getRetryTimes() + 1);
                ScheduledFuture<?> future = taskScheduler.schedule(() -> runTask(n), instant);
                log.debug("getActiveCount: {}", taskScheduler.getActiveCount());
                RETRY_TASKS.put(t.getTaskId(), future);
            }
        } catch (Exception e) {
            log.error(e.toString(), e);
        }
    }

    /**
     * 告警
     *
     * @param t       任务信息
     * @param message 错误消息
     */
    public void alert(AppTaskBo t, String message) {
        String content = t.getAppName() + " " + t.getTaskName() + " 执行异常" + " : " + message;
        sendMessage(t, content);
    }

    /**
     * 发送消息
     * @param t 任务信息
     * @param content 内容
     */
    public void sendMessage(AppTaskBo t, String content) {
        try {
            String robotCode = t.getRobotCode();
            String robotSecret = t.getRobotSecret();
            String openConversationId = t.getOpenConversationId();
            String phones = t.getPhones();
            String userIds = t.getUserIds();
            String alertRobotCode = t.getRobotCode();
            if (TaskAlertTypeEnum.DEFAULT.name().equals(t.getAlertType())) {
                GetAlertConfigVo alertConfig = alertConfigMapper.getConfig();
                if (alertConfig == null) {
                    return;
                }
                robotCode = alertConfig.getRobotCode();
                robotSecret = alertConfig.getRobotSecret();
                openConversationId = alertConfig.getOpenConversationId();
                phones = alertConfig.getPhones();
                userIds = alertConfig.getUserIds();
                alertRobotCode = alertConfig.getRobotCode();
            } else if (TaskAlertTypeEnum.CUSTOM.name().equals(t.getAlertType())) {
                robotCode = t.getAlertRobotCode();
                robotSecret = t.getAlertRobotSecret();
                openConversationId = t.getAlertOpenConversationId();
                phones = t.getAlertPhones();
                userIds = t.getAlertUserIds();
                alertRobotCode = t.getAlertRobotCode();
            }
            if (!StringUtils.hasText(robotCode)
                    || !StringUtils.hasText(robotSecret)
                    || (!StringUtils.hasText(openConversationId)
                    && !StringUtils.hasText(phones)
                    && !StringUtils.hasText(userIds))) {
                return;
            }
            String accessToken = openService.getAccessToken(robotCode, robotSecret);
            if (StringUtils.hasText(openConversationId)) {
                OrgGroupSendResponse response = DingTalkApiUtil.groupSendSampleText(content, accessToken,
                        openConversationId, robotCode);
                iSendMessageLogService.save(new SendMessageLogPo(robotCode, alertRobotCode, openConversationId,
                        content, response));
            }
            if (StringUtils.hasText(phones) || StringUtils.hasText(userIds)) {
                List<String> userIdList = getUserIdList(phones, userIds,
                        accessToken);
                BatchSendOTOResponse response = DingTalkApiUtil.batchSendOtoSampleText(content, accessToken,
                        robotCode, userIdList);
                iSendMessageLogService.save(new SendMessageLogPo(robotCode, alertRobotCode, userIdList,
                        content, response));
            }
        } catch (Exception e) {
            log.error("发送通知失败: {}", e.getMessage(), e);
        }
    }

    /**
     * 匹配文件
     *
     * @param t                任务信息
     * @param template         sftp客户端
     * @param dirPath          文件夹
     * @param fileList         文件列表
     * @param processQueryKeys 消息id列表
     */
    public void findFile(AppTaskBo t, SftpRemoteFileTemplate template, String dirPath, Set<String> fileList,
                         Set<String> processQueryKeys) {
        if (!template.exists(dirPath)) {
            throw new BizException("远端服务器找不到路径" + dirPath);
        }
        SftpClient.DirEntry[] dirEntries = template.list(dirPath);
        log.debug("dirEntries: {}", Arrays.stream(dirEntries).map(SftpClient.DirEntry::getFilename).sorted().toList());
        // 文件夹下所有文件
        if (FileMethodEnum.ALL.name().equals(t.getFileMethod())) {
            List<SftpClient.DirEntry> dirEntryList = Arrays.stream(dirEntries)
                    .filter(tt -> !".".equals(tt.getFilename()) && !"..".equals(tt.getFilename())
                            && tt.getAttributes().isRegularFile()).toList();
            if (CollectionUtils.isEmpty(dirEntryList)) {
                throw new BizException("远端服务器文件夹" + dirPath + "为空");
            }
            dirEntryList.forEach(tt -> sendFile(t, tt.getFilename(), dirPath, template, fileList, processQueryKeys));
        } else if (FileMethodEnum.REG_EXP.name().equals(t.getFileMethod())) {
            // 正则表达式匹配最新文件
            Pattern pattern = Pattern.compile(t.getFilePattern());
            Optional<SftpClient.DirEntry> dirEntryOptional = Arrays.stream(dirEntries)
                    .filter(tt -> !".".equals(tt.getFilename()) && !"..".equals(tt.getFilename())
                            && tt.getAttributes().isRegularFile() && pattern.matcher(tt.getFilename()).find())
                    .max(Comparator.comparing(tt -> tt.getAttributes().getModifyTime()));
            SftpClient.DirEntry dirEntry = dirEntryOptional.orElseThrow(() -> new BizException("远端服务器文件夹"
                    + dirPath + "下没有找到匹配的文件"));
            sendFile(t, dirEntry.getFilename(), dirPath, template, fileList, processQueryKeys);
        } else if (FileMethodEnum.MONTH.name().equals(t.getFileMethod())) {
            // 按月匹配文件
            LocalDate localDate;
            if (StringUtils.hasText(t.getDateString())) {
                localDate = dateStringParse(t);
            } else {
                localDate = LocalDate.now().plusMonths(t.getFileTimeDelay());
            }
            String month = localDate.format(DateTimeFormatter.ofPattern("yyyyMM"));
            String filename = t.getFilePrefix() + month + "." + t.getFileExtension();
            sendFile(t, filename, dirPath, template, fileList, processQueryKeys);
        } else if (FileMethodEnum.DAY.name().equals(t.getFileMethod())) {
            // 按日匹配文件
            LocalDate localDate;
            if (StringUtils.hasText(t.getDateString())) {
                localDate = dateStringParse(t);
            } else {
                localDate = LocalDate.now().plusDays(t.getFileTimeDelay());
            }
            String day = localDate.format(DateTimeFormatter.ofPattern("yyyyMMdd"));
            String filename = t.getFilePrefix() + day + "." + t.getFileExtension();
            sendFile(t, filename, dirPath, template, fileList, processQueryKeys);
        }
    }

    /**
     * 发送文件
     *
     * @param t                任务信息
     * @param filename         文件名
     * @param dirPath          文件夹
     * @param template         sftp客户端
     * @param fileList         文件列表
     * @param processQueryKeys 消息id列表
     */
    public void sendFile(AppTaskBo t, String filename, String dirPath, SftpRemoteFileTemplate template,
                         Set<String> fileList, Set<String> processQueryKeys) {
        String filePath = dirPath + (dirPath.endsWith("/") ? "" : "/") + filename;
        fileList.add(filePath);
        if (!template.exists(filePath)) {
            throw new BizException("远端服务器找不到文件" + filePath);
        }
        // 检查文件大小
        long size = template.list(filePath)[0].getAttributes().getSize();
        OpenService.checkFileSize(size);
        byte[] bytes;
        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            template.execute((SessionCallbackWithoutResult<SftpClient.DirEntry>) session
                    -> session.read(filePath, os));
            bytes = os.toByteArray();
        } catch (IOException e) {
            throw new MyRuntimeException(e);
        }
        String fileExtension = OpenService.getFileExtension(filename);
        String type = OpenService.getUploadType(fileExtension);
        String robotCode = t.getRobotCode();
        String openConversationId = t.getOpenConversationId();
        String accessToken = openService.getAccessToken(robotCode, t.getRobotSecret());
        String alertRobotCode = null;
        if (TaskAlertTypeEnum.TASK.name().equals(t.getAlertType())) {
            alertRobotCode = robotCode;
        } else if (TaskAlertTypeEnum.CUSTOM.name().equals(t.getAlertType())) {
            alertRobotCode = t.getAlertRobotCode();
        }
        // 发送markdown消息
        if (t.getFileToText().equals(1) && ("txt".equalsIgnoreCase(fileExtension) || "md".equalsIgnoreCase(fileExtension))) {
            String content = new String(bytes, StandardCharsets.UTF_8);
            // 发送单聊消息
            if (t.getConversationType().equals(1)) {
                List<String> userIdList = getUserIdList(t.getPhones(), t.getUserIds(), accessToken);
                BatchSendOTOResponse r = DingTalkApiUtil.batchSendOtoSampleMarkdown(
                        org.apache.commons.lang3.StringUtils.substring(content, 0, 30), content,
                        accessToken, robotCode, userIdList);
                processQueryKeys.add(r.body.processQueryKey);
                iSendMessageLogService.save(new SendMessageLogPo(robotCode, alertRobotCode, userIdList, content, r));
            } else {
                // 发送群聊消息
                OrgGroupSendResponse r = DingTalkApiUtil.groupSendSampleMarkdown(
                        org.apache.commons.lang3.StringUtils.substring(content, 0, 30), content,
                        accessToken, openConversationId, robotCode);
                processQueryKeys.add(r.body.processQueryKey);
                iSendMessageLogService.save(new SendMessageLogPo(robotCode, alertRobotCode, openConversationId,
                        content, r));
            }
            return;
        }
        // 上传文件
        FileItem fileItem = new FileItem(filename, bytes);
        OapiMediaUploadResponse uploadResponse = DingTalkApiUtil.upload(type, fileItem, accessToken);
        String mediaId = uploadResponse.getMediaId();
        // 发送单聊消息
        if (t.getConversationType().equals(1)) {
            List<String> userIdList = getUserIdList(t.getPhones(), t.getUserIds(), accessToken);
            if ("image".equals(type)) {
                // 发送图片
                BatchSendOTOResponse r = DingTalkApiUtil.batchSendOtoSampleImageMsg(mediaId, accessToken, robotCode,
                        userIdList);
                processQueryKeys.add(r.body.processQueryKey);
                iSendMessageLogService.save(new SendMessageLogPo(robotCode, alertRobotCode, userIdList, mediaId, r));
            } else {
                // 发送文件
                BatchSendOTOResponse r = DingTalkApiUtil.batchSendOtoSampleFile(mediaId, filename, fileExtension,
                        accessToken, robotCode, userIdList);
                processQueryKeys.add(r.body.processQueryKey);
                iSendMessageLogService.save(new SendMessageLogPo(robotCode, alertRobotCode, userIdList, filename, r));
            }
        } else {
            // 发送群聊消息
            if ("image".equals(type)) {
                // 发送图片
                OrgGroupSendResponse r = DingTalkApiUtil.groupSendSampleImageMsg(mediaId, accessToken,
                        openConversationId, robotCode);
                processQueryKeys.add(r.body.processQueryKey);
                iSendMessageLogService.save(new SendMessageLogPo(robotCode, alertRobotCode, openConversationId,
                        mediaId, r));
            } else {
                // 发送文件
                OrgGroupSendResponse r = DingTalkApiUtil.groupSendSampleFile(mediaId, filename, fileExtension,
                        accessToken, openConversationId, robotCode);
                processQueryKeys.add(r.body.processQueryKey);
                iSendMessageLogService.save(new SendMessageLogPo(robotCode, alertRobotCode, openConversationId,
                        filename, r));
            }
        }
    }

    /**
     * 获取钉钉用户id
     *
     * @param phones 手机号
     * @param userIds 用户id
     * @param accessToken 钉钉接口访问凭证
     */
    public List<String> getUserIdList(String phones, String userIds, String accessToken) {
        Set<String> phonesSet = new HashSet<>();
        if (StringUtils.hasText(phones)) {
            phonesSet = Arrays.stream(phones.split(",")).collect(Collectors.toSet());
        }
        Set<String> userIdsSet = new HashSet<>();
        if (StringUtils.hasText(userIds)) {
            userIdsSet = Arrays.stream(userIds.split(",")).collect(Collectors.toSet());
        }
        openService.checkPhonesUserIds(phonesSet, userIdsSet);
        Map<String, String> failPhones = new HashMap<>();
        return openService.getUserIds(userIdsSet, phonesSet, accessToken, failPhones);
    }

    /**
     * 检查任务参数
     * @param dto 任务参数
     */
    public static void checkAppTaskDto(AddAppTaskDto dto) {
        if (dto.getConversationType().equals(1)) {
            if (!StringUtils.hasText(dto.getPhones()) && !StringUtils.hasText(dto.getUserIds())) {
                throw new BizException("phones或userIds必须有一个不能为空");
            }
            if (StringUtils.hasText(dto.getPhones())) {
                Matcher matcher = PHONE_LIST_PATTERN.matcher(dto.getPhones());
                if (!matcher.matches()) {
                    throw new BizException("phones格式不正确");
                }
            } else {
                dto.setPhones("");
            }
            if (StringUtils.hasText(dto.getUserIds())) {
                Matcher matcher = USER_ID_LIST_PATTERN.matcher(dto.getUserIds());
                if (!matcher.matches()) {
                    throw new BizException("userIds格式不正确");
                }
            } else {
                dto.setUserIds("");
            }
            dto.setOpenConversationId("");
        } else {
            if (!StringUtils.hasText(dto.getOpenConversationId())) {
                throw new BizException("openConversationId不能为空");
            }
            dto.setPhones("");
            dto.setUserIds("");
        }
        if (dto.getHasSubdirectory().equals(1)) {
            if (dto.getSubdirectoryMethod() == null) {
                throw new BizException("subdirectoryMethod不能为空");
            }
            if (SubdirectoryMethodEnum.REG_EXP.equals(dto.getSubdirectoryMethod())) {
                if (!StringUtils.hasText(dto.getSubdirectoryPattern())) {
                    throw new BizException("subdirectoryPattern不能为空");
                }
                dto.setDirTimeDelay(0);
            } else {
                dto.setSubdirectoryPattern("");
            }
        } else {
            dto.setSubdirectoryMethod(null);
            dto.setSubdirectoryPattern("");
            dto.setDirTimeDelay(0);
        }
        if (FileMethodEnum.DAY.equals(dto.getFileMethod()) || FileMethodEnum.MONTH.equals(dto.getFileMethod())) {
            if (dto.getFileTimeDelay() == null) {
                throw new BizException("fileMinusTime不能为空");
            }
            if (!StringUtils.hasText(dto.getFilePrefix())) {
                throw new BizException("filePrefix不能为空");
            }
            if (!StringUtils.hasText(dto.getFileExtension())) {
                throw new BizException("fileExtension不能为空");
            }
            dto.setFilePattern("");
        }
        if (FileMethodEnum.REG_EXP.equals(dto.getFileMethod())) {
            if (!StringUtils.hasText(dto.getFilePattern())) {
                throw new BizException("filePattern不能为空");
            }
            dto.setFilePrefix("");
            dto.setFileExtension("");
            dto.setFileTimeDelay(0);
        }
        if (TaskAlertTypeEnum.CUSTOM.equals(dto.getAlertType())) {
            if (StringUtils.hasText(dto.getAlertPhones())) {
                Matcher matcher = PHONE_LIST_PATTERN.matcher(dto.getAlertPhones());
                if (!matcher.matches()) {
                    throw new BizException("alertPhones格式不正确");
                }
            } else {
                dto.setAlertPhones("");
            }
            if (StringUtils.hasText(dto.getAlertUserIds())) {
                Matcher matcher = USER_ID_LIST_PATTERN.matcher(dto.getAlertUserIds());
                if (!matcher.matches()) {
                    throw new BizException("alertUserIds格式不正确");
                }
            } else {
                dto.setAlertUserIds("");
            }
            if (!StringUtils.hasText(dto.getAlertRobotCode())) {
                dto.setAlertRobotCode("");
            }
            if (!StringUtils.hasText(dto.getAlertOpenConversationId())) {
                dto.setAlertOpenConversationId("");
            }
        } else {
            dto.setAlertRobotCode("");
            dto.setAlertOpenConversationId("");
            dto.setAlertPhones("");
            dto.setAlertUserIds("");
        }
    }

    public R<PageVo<ListAppTaskVo>> listAppTask(ListAppTaskDto dto) {
        Page<AppTaskBo> page = new Page<>(dto.getCurrent(), dto.getPageSize());
        if (CollectionUtils.isEmpty(dto.getSort())) {
            page.addOrder(OrderItem.desc("update_time"));
        } else {
            List<OrderItem> items = dto.getSort().entrySet().stream()
                    .map(t -> t.getValue().equals(SortEnum.ascend) ? OrderItem.asc(t.getKey().value)
                            : OrderItem.desc(t.getKey().value)).toList();
            page.addOrder(items);
        }
        List<AppTaskBo> list = appTaskMapper.listAppTask(page, dto);
        List<ListAppTaskVo> vos = list.stream().map(ListAppTaskVo::new).toList();
        return R.ok(new PageVo<>(page.getTotal(), vos));
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> addAppTask(AddAppTaskDto dto) {
        checkAppTaskDto(dto);
        QueryWrapper<AppTaskPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", dto.getAppId());
        wrapper.eq("task_name", dto.getTaskName());
        wrapper.ne("status", AppTaskStatusEnum.DELETED.value);
        boolean exists = iAppTaskService.exists(wrapper);
        if (exists) {
            throw new BizException("任务名称已存在");
        }
        AppTaskPo po = new AppTaskPo(dto);
        iAppTaskService.save(po);
        AppTaskBo bo = appTaskMapper.getAppTaskByTaskId(po.getTaskId());
        if (StringUtils.hasText(bo.getCron())) {
            addTask(bo);
        }
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> updateAppTask(UpdateAppTaskDto dto) {
        checkAppTaskDto(dto);
        QueryWrapper<AppTaskPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", dto.getAppId());
        wrapper.eq("task_name", dto.getTaskName());
        wrapper.ne("status", AppTaskStatusEnum.DELETED.value);
        wrapper.ne("task_id", dto.getTaskId());
        boolean exists = iAppTaskService.exists(wrapper);
        if (exists) {
            throw new BizException("任务名称已存在");
        }
        AppTaskPo po = new AppTaskPo(dto);
        iAppTaskService.updateById(po);
        AppTaskBo bo = appTaskMapper.getAppTaskByTaskId(po.getTaskId());
        if (AppTaskStatusEnum.RUNNING.value.equals(bo.getStatus()) && StringUtils.hasText(bo.getCron())) {
            addTask(bo);
        }
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> enableAppTask(EnableAppTaskDto dto) {
        AppTaskPo po = new AppTaskPo();
        po.setTaskId(dto.getTaskId());
        po.setStatus(AppTaskStatusEnum.RUNNING.value);
        iAppTaskService.updateById(po);
        AppTaskBo bo = appTaskMapper.getAppTaskByTaskId(po.getTaskId());
        if (!StringUtils.hasText(bo.getCron())) {
            throw new BizException("cron表达式未设置无法启用");
        }
        addTask(bo);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> stopAppTask(StopAppTaskDto dto) {
        AppTaskPo po = new AppTaskPo();
        po.setTaskId(dto.getTaskId());
        po.setStatus(AppTaskStatusEnum.STOPPED.value);
        iAppTaskService.updateById(po);
        removeTask(dto.getTaskId());
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> deleteAppTask(DeleteAppTaskDto dto) {
        AppTaskPo po = new AppTaskPo();
        po.setTaskId(dto.getTaskId());
        po.setStatus(AppTaskStatusEnum.DELETED.value);
        iAppTaskService.updateById(po);
        removeTask(dto.getTaskId());
        return R.ok();
    }

    public R<Object> runAppTask(RunAppTaskDto dto) {
        AppTaskBo bo = appTaskMapper.getAppTaskByTaskId(dto.getTaskId());
        bo.setDateString(dto.getDateString());
        runTask(bo);
        return R.ok();
    }

    public R<PageVo<AppTaskLogPo>> listAppTaskLog(ListAppTaskLogDto dto) {
        QueryWrapper<AppTaskLogPo> wrapper = new QueryWrapper<>();
        if (StringUtils.hasText(dto.getId())) {
            wrapper.eq("id", dto.getId());
        }
        if (StringUtils.hasText(dto.getAppId())) {
            wrapper.eq("app_id", dto.getAppId());
        }
        if (StringUtils.hasText(dto.getAppName())) {
            wrapper.like("app_name", "%" + dto.getAppName() + "%");
        }
        if (StringUtils.hasText(dto.getTaskId())) {
            wrapper.eq("task_id", dto.getTaskId());
        }
        if (StringUtils.hasText(dto.getTaskName())) {
            wrapper.like("task_name", "%" + dto.getTaskName() + "%");
        }
        if (dto.getStatus() != null) {
            wrapper.eq("status", dto.getStatus());
        }
        if (StringUtils.hasText(dto.getRobotCode())) {
            wrapper.eq("robot_code", dto.getRobotCode());
        }
        if (StringUtils.hasText(dto.getRobotName())) {
            wrapper.like("robot_name", "%" + dto.getRobotName() + "%");
        }
        if (dto.getConversationType() != null) {
            wrapper.eq("conversation_type", dto.getConversationType());
        }
        if (StringUtils.hasText(dto.getOpenConversationId())) {
            wrapper.eq("open_conversation_id", dto.getOpenConversationId());
        }
        if (StringUtils.hasText(dto.getPhones())) {
            wrapper.like("phones", "%" + dto.getPhones() + "%");
        }
        if (StringUtils.hasText(dto.getUserIds())) {
            wrapper.like("user_ids", "%" + dto.getUserIds() + "%");
        }
        if (StringUtils.hasText(dto.getSftpId())) {
            wrapper.eq("sftp_id", dto.getSftpId());
        }
        if (StringUtils.hasText(dto.getHost())) {
            wrapper.like("host", "%" + dto.getHost() + "%");
        }
        if (dto.getStartTime() != null) {
            wrapper.ge("create_time", dto.getStartTime());
        }
        if (dto.getEndTime() != null) {
            wrapper.le("create_time", dto.getEndTime());
        }
        Page<AppTaskLogPo> page = new Page<>(dto.getCurrent(), dto.getPageSize());
        if (CollectionUtils.isEmpty(dto.getSort())) {
            page.addOrder(OrderItem.desc("create_time"));
        } else {
            List<OrderItem> items = dto.getSort().entrySet().stream()
                    .map(t -> t.getValue().equals(SortEnum.ascend) ? OrderItem.asc(t.getKey().value)
                            : OrderItem.desc(t.getKey().value)).toList();
            page.addOrder(items);
        }
        List<AppTaskLogPo> vos = iAppTaskLogService.list(page, wrapper);
        return R.ok(new PageVo<>(page.getTotal(), vos));
    }
}
