package com.nokia.dingtalk_api.service;

import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponse;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponseBody;
import com.aliyun.dingtalkrobot_1_0.models.BatchRecallOTOResponse;
import com.aliyun.dingtalkrobot_1_0.models.BatchSendOTOResponse;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupRecallResponse;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupSendResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dingtalk.api.response.OapiMediaUploadResponse;
import com.dingtalk.api.response.OapiV2UserGetbymobileResponse;
import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.common.exception.BizException;
import com.nokia.dingtalk_api.common.exception.MyRuntimeException;
import com.nokia.dingtalk_api.dao.IAppSftpService;
import com.nokia.dingtalk_api.dao.ISendMessageLogService;
import com.nokia.dingtalk_api.mapper.AppRobotMapper;
import com.nokia.dingtalk_api.pojos.dto.open.BatchRecallOtoMessagesDto;
import com.nokia.dingtalk_api.pojos.dto.open.BatchSendOtoSampleFileDto;
import com.nokia.dingtalk_api.pojos.dto.open.BatchSendOtoSampleImageMsgDto;
import com.nokia.dingtalk_api.pojos.dto.open.BatchSendSampleMarkdownDto;
import com.nokia.dingtalk_api.pojos.dto.open.BatchSendSampleTextDto;
import com.nokia.dingtalk_api.pojos.dto.open.GetUserIdByMobileDto;
import com.nokia.dingtalk_api.pojos.dto.open.GroupSendSampleFileDto;
import com.nokia.dingtalk_api.pojos.dto.open.GroupSendSampleImageMsgDto;
import com.nokia.dingtalk_api.pojos.dto.open.GroupSendSampleMarkdownDto;
import com.nokia.dingtalk_api.pojos.dto.open.GroupSendSampleTextDto;
import com.nokia.dingtalk_api.pojos.dto.open.RecallGroupMessagesDto;
import com.nokia.dingtalk_api.pojos.dto.open.SftpBatchSendOtoSampleFileDto;
import com.nokia.dingtalk_api.pojos.dto.open.SftpBatchSendOtoSampleImageMsgDto;
import com.nokia.dingtalk_api.pojos.dto.open.SftpGroupSendSampleFileDto;
import com.nokia.dingtalk_api.pojos.dto.open.SftpGroupSendSampleImageMsgDto;
import com.nokia.dingtalk_api.pojos.enums.RedisPrefixEnum;
import com.nokia.dingtalk_api.pojos.po.AppSftpPo;
import com.nokia.dingtalk_api.pojos.po.RobotPo;
import com.nokia.dingtalk_api.pojos.po.SendMessageLogPo;
import com.nokia.dingtalk_api.pojos.vo.open.BatchRecallOtoMessagesVo;
import com.nokia.dingtalk_api.pojos.vo.open.BatchSendVo;
import com.nokia.dingtalk_api.pojos.vo.open.GetUserIdByMobileVo;
import com.nokia.dingtalk_api.pojos.vo.open.GroupSendVo;
import com.nokia.dingtalk_api.pojos.vo.open.RecallGroupMessagesVo;
import com.nokia.dingtalk_api.pojos.vo.open.UploadVo;
import com.nokia.dingtalk_api.util.DingTalkApiUtil;
import com.taobao.api.FileItem;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.sshd.sftp.client.SftpClient;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.integration.file.remote.SessionCallbackWithoutResult;
import org.springframework.integration.sftp.session.DefaultSftpSessionFactory;
import org.springframework.integration.sftp.session.SftpRemoteFileTemplate;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.Base64;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Stream;

@Slf4j
@Service
@RequiredArgsConstructor
public class OpenService {
    static final List<String> IMAGE_TYPES = Stream.of("jpg", "gif", "png", "bmp").toList();
    private final AppRobotMapper appRobotMapper;
    private final IAppSftpService iAppSftpService;
    private final AesService aesService;
    private final RedisTemplate<String, Object> redisTemplate;
    private final ISendMessageLogService iSendMessageLogService;

    /**
     * 获取当前请求的appId
     */
    public String getAppId() {
        SecurityContext context = SecurityContextHolder.getContext();
        Authentication authentication = context.getAuthentication();
        return (String) authentication.getPrincipal();
    }

    /**
     * 获取企业内部应用的accessToken
     * @param appKey 已创建的企业内部应用的AppKey
     * @param appSecret 已创建的企业内部应用的AppSecret
     */
    public String getAccessToken(String appKey, String appSecret) {
        String k = RedisPrefixEnum.DINGTALK_ACCESS_TOKEN.prefix + appKey;
        GetAccessTokenResponseBody o = (GetAccessTokenResponseBody) redisTemplate.opsForValue().get(k);
        if (o != null) {
            return o.accessToken;
        }
        GetAccessTokenResponse r = DingTalkApiUtil.getAccessToken(appKey, aesService.decrypt(appSecret));
        redisTemplate.opsForValue().set(RedisPrefixEnum.DINGTALK_ACCESS_TOKEN.prefix + appKey,
                r.body, r.body.expireIn - 60, RedisPrefixEnum.DINGTALK_ACCESS_TOKEN.timeUnit);
        return r.body.accessToken;
    }

    /**
     * 获取企业内部应用的accessToken
     * @param appKey 已创建的企业内部应用的AppKey
     */
    public String getAccessToken(String appKey) {
        String appId = getAppId();
        RobotPo robotPo = appRobotMapper.getRobotByAppIdAndRobotCode(appId, appKey);
        if (robotPo == null) {
            throw new BizException("该机器人未添加");
        }
        return getAccessToken(appKey, robotPo.getRobotSecret());
    }

    /**
     * 检查文件大小
     * @param size 文件大小
     */
    public static void checkFileSize(long size) {
        if (size == 0) {
            throw new BizException("文件为空");
        }
        if (size > 20971520) {
            throw new BizException("文件最大20MB");
        }
    }

    /**
     * 获取上传类型
     * @param fileExtension 文件类型
     */
    public static String getUploadType(String fileExtension) {
        if (IMAGE_TYPES.stream().anyMatch(t -> t.equalsIgnoreCase(fileExtension))) {
            return "image";
        }
        return "file";
    }

    /**
     * 获取文件类型
     * @param filename 文件名
     */
    public static String getFileExtension(String filename) {
        if (!StringUtils.hasText(filename)) {
            return "file";
        }
        int index = filename.lastIndexOf('.');
        if (index > 0 && index < filename.length() - 1) {
            return filename.substring(index + 1);
        } else {
            return "file";
        }
    }

    /**
     * 获取userId
     */
    public List<String> getUserIds(Set<String> userIds, Set<String> phones, String accessToken, Map<String, String> failPhones) {
        Set<String> set = new HashSet<>();
        if (!CollectionUtils.isEmpty(phones)) {
            phones.forEach(t -> {
                try {
                    OapiV2UserGetbymobileResponse response = DingTalkApiUtil.getUserByMobile(accessToken, t);
                    if (response.isSuccess()) {
                        set.add(response.getResult().getUserid());
                    } else {
                        failPhones.put(t, response.getErrmsg());
                    }
                } catch (Exception e) {
                    log.error(e.toString(), e);
                    failPhones.put(t, "未知错误");
                }
            });
        }
        if (!CollectionUtils.isEmpty(userIds)) {
            set.addAll(userIds);
        }
        if (CollectionUtils.isEmpty(set)) {
            throw new BizException(failPhones.toString());
        }
        return set.stream().toList();
    }

    /**
     * 检查接收机器人消息的用户的手机号列表和userIds列表
     * @param phones 接收机器人消息的用户的手机号列表
     * @param userIds 接收机器人消息的用户的userId列表
     */
    public void checkPhonesUserIds(Set<String> phones, Set<String> userIds) {
        if (CollectionUtils.isEmpty(phones) && CollectionUtils.isEmpty(userIds)) {
            throw new BizException("phones或userIds必须有一个不能为空");
        }
    }

    /**
     * 下载sftp文件
     * @param sftpId sftpId
     * @param filePath 文件路径
     */
    public byte[] getSftpFile(String sftpId, String filePath) {
        String appId = getAppId();
        QueryWrapper<AppSftpPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", appId);
        wrapper.eq("sftp_id", sftpId);
        AppSftpPo appSftpPo = iAppSftpService.getOne(wrapper);
        if (appSftpPo == null) {
            throw new BizException("sftpId错误");
        }
        DefaultSftpSessionFactory factory = new DefaultSftpSessionFactory();
        factory.setHost(appSftpPo.getHost());
        factory.setPort(appSftpPo.getPort());
        factory.setUser(appSftpPo.getUsername());
        factory.setPassword(appSftpPo.getPassword());
        factory.setAllowUnknownKeys(true);
        SftpRemoteFileTemplate template = new SftpRemoteFileTemplate(factory);
        boolean exists = template.exists(filePath);
        if (!exists) {
            throw new BizException("找不到" + filePath);
        }
        byte[] bytes;
        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            template.execute((SessionCallbackWithoutResult<SftpClient.DirEntry>) session -> session.read(filePath, os));
            bytes = os.toByteArray();
        } catch (Exception e) {
            throw new MyRuntimeException(e);
        }
        checkFileSize(bytes.length);
        return bytes;
    }

    public R<GroupSendVo> groupSendSampleText(GroupSendSampleTextDto dto) {
        String accessToken = getAccessToken(dto.getRobotCode());
        OrgGroupSendResponse response = DingTalkApiUtil.groupSendSampleText(dto.getContent(), accessToken,
                dto.getOpenConversationId(), dto.getRobotCode());
        iSendMessageLogService.save(new SendMessageLogPo(dto.getRobotCode(), dto.getOpenConversationId(),
                dto.getContent(), response));
        return R.ok(new GroupSendVo(response));
    }

    public R<GroupSendVo> groupSendSampleMarkdown(GroupSendSampleMarkdownDto dto) {
        String accessToken = getAccessToken(dto.getRobotCode());
        OrgGroupSendResponse response = DingTalkApiUtil.groupSendSampleMarkdown(dto.getTitle(), dto.getText(),
                accessToken, dto.getOpenConversationId(), dto.getRobotCode());
        iSendMessageLogService.save(new SendMessageLogPo(dto.getRobotCode(), dto.getOpenConversationId(),
                dto.getText(), response));
        return R.ok(new GroupSendVo(response));
    }

    public R<GroupSendVo> groupSendSampleImageMsg(GroupSendSampleImageMsgDto dto) {
        String accessToken = getAccessToken(dto.getRobotCode());
        OrgGroupSendResponse response = DingTalkApiUtil.groupSendSampleImageMsg(dto.getMediaId(), accessToken,
                dto.getOpenConversationId(), dto.getRobotCode());
        iSendMessageLogService.save(new SendMessageLogPo(dto.getRobotCode(), dto.getOpenConversationId(),
                dto.getMediaId(), response));
        return R.ok(new GroupSendVo(response));
    }

    public R<GroupSendVo> groupSendSampleFile(GroupSendSampleFileDto dto) {
        String fileExtension = getFileExtension(dto.getFilename());
        String accessToken = getAccessToken(dto.getRobotCode());
        OrgGroupSendResponse response = DingTalkApiUtil.groupSendSampleFile(dto.getMediaId(), dto.getFilename(),
                fileExtension, accessToken, dto.getOpenConversationId(), dto.getRobotCode());
        iSendMessageLogService.save(new SendMessageLogPo(dto.getRobotCode(), dto.getOpenConversationId(),
                dto.getFilename(), response));
        return R.ok(new GroupSendVo(response));
    }

    public R<BatchSendVo> batchSendOtoSampleText(BatchSendSampleTextDto dto) {
        checkPhonesUserIds(dto.getPhones(), dto.getUserIds());
        String accessToken = getAccessToken(dto.getRobotCode());
        Map<String, String> failPhones = new HashMap<>();
        List<String> userIds = getUserIds(dto.getUserIds(), dto.getPhones(), accessToken, failPhones);
        BatchSendOTOResponse response = DingTalkApiUtil.batchSendOtoSampleText(dto.getContent(), accessToken,
                dto.getRobotCode(), userIds);
        iSendMessageLogService.save(new SendMessageLogPo(dto.getRobotCode(), userIds, dto.getContent(), response));
        return R.ok(new BatchSendVo(response, failPhones));
    }

    public R<BatchSendVo> batchSendOtoSampleMarkdown(BatchSendSampleMarkdownDto dto) {
        checkPhonesUserIds(dto.getPhones(), dto.getUserIds());
        String accessToken = getAccessToken(dto.getRobotCode());
        Map<String, String> failPhones = new HashMap<>();
        List<String> userIds = getUserIds(dto.getUserIds(), dto.getPhones(), accessToken, failPhones);
        BatchSendOTOResponse response = DingTalkApiUtil.batchSendOtoSampleMarkdown(dto.getTitle(), dto.getText(), accessToken,
                dto.getRobotCode(), userIds);
        iSendMessageLogService.save(new SendMessageLogPo(dto.getRobotCode(), userIds, dto.getText(), response));
        return R.ok(new BatchSendVo(response, failPhones));
    }

    public R<BatchSendVo> batchSendOtoSampleImageMsg(BatchSendOtoSampleImageMsgDto dto) {
        checkPhonesUserIds(dto.getPhones(), dto.getUserIds());
        String accessToken = getAccessToken(dto.getRobotCode());
        Map<String, String> failPhones = new HashMap<>();
        List<String> userIds = getUserIds(dto.getUserIds(), dto.getPhones(), accessToken, failPhones);
        BatchSendOTOResponse response = DingTalkApiUtil.batchSendOtoSampleImageMsg(dto.getMediaId(), accessToken,
                dto.getRobotCode(), userIds);
        iSendMessageLogService.save(new SendMessageLogPo(dto.getRobotCode(), userIds, dto.getMediaId(), response));
        return R.ok(new BatchSendVo(response, failPhones));
    }

    public R<BatchSendVo> batchSendOtoSampleFile(BatchSendOtoSampleFileDto dto) {
        checkPhonesUserIds(dto.getPhones(), dto.getUserIds());
        String accessToken = getAccessToken(dto.getRobotCode());
        Map<String, String> failPhones = new HashMap<>();
        List<String> userIds = getUserIds(dto.getUserIds(), dto.getPhones(), accessToken, failPhones);
        String fileExtension = getFileExtension(dto.getFilename());
        BatchSendOTOResponse response = DingTalkApiUtil.batchSendOtoSampleFile(dto.getMediaId(), dto.getFilename(),
                fileExtension, accessToken, dto.getRobotCode(), userIds);
        iSendMessageLogService.save(new SendMessageLogPo(dto.getRobotCode(), userIds, dto.getFilename(), response));
        return R.ok(new BatchSendVo(response, failPhones));
    }

    public R<BatchRecallOtoMessagesVo> batchRecallOtoMessages(BatchRecallOtoMessagesDto dto) {
        String accessToken = getAccessToken(dto.getRobotCode());
        BatchRecallOTOResponse response = DingTalkApiUtil.batchRecallOtoMessages(accessToken, dto.getRobotCode(),
                dto.getProcessQueryKeys().stream().toList());
        return R.ok(new BatchRecallOtoMessagesVo(response));
    }

    public R<RecallGroupMessagesVo> recallGroupMessages(RecallGroupMessagesDto dto) {
        String accessToken = getAccessToken(dto.getRobotCode());
        OrgGroupRecallResponse response = DingTalkApiUtil.recallGroupMessages(accessToken, dto.getRobotCode(),
                dto.getOpenConversationId(), dto.getProcessQueryKeys().stream().toList());
        return R.ok(new RecallGroupMessagesVo(response));
    }

    public R<GetUserIdByMobileVo> getUserIdByMobile(GetUserIdByMobileDto dto) {
        String accessToken = getAccessToken(dto.getRobotCode());
        try {
            OapiV2UserGetbymobileResponse response = DingTalkApiUtil.getUserByMobile(accessToken, dto.getMobile());
            if (response.isSuccess()) {
                return R.ok(new GetUserIdByMobileVo(response));
            } else {
                return R.error(response.getErrmsg());
            }
        } catch (Exception e) {
            log.error(e.toString(), e);
            return R.error();
        }
    }

    public R<UploadVo> upload(String robotCode, MultipartFile file) {
        checkFileSize(file.getSize());
        String filename = file.getOriginalFilename();
        String fileExtension = getFileExtension(filename);
        String type = getUploadType(fileExtension);
        FileItem fileItem;
        try {
            fileItem = new FileItem(filename, file.getInputStream());
        } catch (IOException e) {
            throw new MyRuntimeException(e);
        }
        String robotCodeDecode = new String(Base64.getDecoder().decode(robotCode));
        String accessToken = getAccessToken(robotCodeDecode);
        OapiMediaUploadResponse response = DingTalkApiUtil.upload(type, fileItem, accessToken);
        return R.ok(new UploadVo(response));
    }

    public R<GroupSendVo> sftpGroupSendSampleImageMsg(SftpGroupSendSampleImageMsgDto dto) {
        byte[] bytes = getSftpFile(dto.getSftpId(), dto.getFilePath());
        File file = new File(dto.getFilePath());
        String filename = file.getName();
        String fileExtension = getFileExtension(filename);
        String type = getUploadType(fileExtension);
        FileItem fileItem = new FileItem(filename, bytes);
        String accessToken = getAccessToken(dto.getRobotCode());
        OapiMediaUploadResponse uploadResponse = DingTalkApiUtil.upload(type, fileItem, accessToken);
        OrgGroupSendResponse response = DingTalkApiUtil.groupSendSampleImageMsg(uploadResponse.getMediaId(),
                accessToken, dto.getOpenConversationId(), dto.getRobotCode());
        return R.ok(new GroupSendVo(response));
    }

    public R<GroupSendVo> sftpGroupSendSampleFile(SftpGroupSendSampleFileDto dto) {
        byte[] bytes = getSftpFile(dto.getSftpId(), dto.getFilePath());
        File file = new File(dto.getFilePath());
        String filename = file.getName();
        String fileExtension = getFileExtension(filename);
        String type = getUploadType(fileExtension);
        FileItem fileItem = new FileItem(filename, bytes);
        String accessToken = getAccessToken(dto.getRobotCode());
        OapiMediaUploadResponse uploadResponse = DingTalkApiUtil.upload(type, fileItem, accessToken);
        OrgGroupSendResponse response = DingTalkApiUtil.groupSendSampleFile(uploadResponse.getMediaId(), filename,
                fileExtension, accessToken, dto.getOpenConversationId(), dto.getRobotCode());
        return R.ok(new GroupSendVo(response));
    }

    public R<BatchSendVo> sftpBatchSendOtoSampleImageMsg(SftpBatchSendOtoSampleImageMsgDto dto) {
        checkPhonesUserIds(dto.getPhones(), dto.getUserIds());
        byte[] bytes = getSftpFile(dto.getSftpId(), dto.getFilePath());
        File file = new File(dto.getFilePath());
        String filename = file.getName();
        String fileExtension = getFileExtension(filename);
        String type = getUploadType(fileExtension);
        FileItem fileItem = new FileItem(filename, bytes);
        String accessToken = getAccessToken(dto.getRobotCode());
        OapiMediaUploadResponse uploadResponse = DingTalkApiUtil.upload(type, fileItem, accessToken);
        return batchSendOtoSampleImageMsg(new BatchSendOtoSampleImageMsgDto(dto, uploadResponse.getMediaId()));
    }

    public R<BatchSendVo> sftpBatchSendOtoSampleFile(SftpBatchSendOtoSampleFileDto dto) {
        checkPhonesUserIds(dto.getPhones(), dto.getUserIds());
        byte[] bytes = getSftpFile(dto.getSftpId(), dto.getFilePath());
        File file = new File(dto.getFilePath());
        String filename = file.getName();
        String fileExtension = getFileExtension(filename);
        String type = getUploadType(fileExtension);
        FileItem fileItem = new FileItem(filename, bytes);
        String accessToken = getAccessToken(dto.getRobotCode());
        OapiMediaUploadResponse uploadResponse = DingTalkApiUtil.upload(type, fileItem, accessToken);
        return batchSendOtoSampleFile(new BatchSendOtoSampleFileDto(dto, uploadResponse.getMediaId(), filename));
    }
}
