package com.nokia.dingtalk_api.service;

import cn.hutool.core.util.IdUtil;
import com.aliyun.dingtalkrobot_1_0.models.BatchRecallOTOResponse;
import com.aliyun.dingtalkrobot_1_0.models.BatchSendOTOResponse;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupRecallResponse;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupSendResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dingtalk.open.app.api.callback.OpenDingTalkCallbackListener;
import com.nokia.dingtalk_api.dao.IAppTaskService;
import com.nokia.dingtalk_api.dao.IReceiveMessageLogService;
import com.nokia.dingtalk_api.dao.IRobotCommandService;
import com.nokia.dingtalk_api.dao.IRobotService;
import com.nokia.dingtalk_api.dao.ISendMessageLogService;
import com.nokia.dingtalk_api.mapper.AlertConfigMapper;
import com.nokia.dingtalk_api.mapper.AppRobotMapper;
import com.nokia.dingtalk_api.pojos.dto.cms.RunAppTaskDto;
import com.nokia.dingtalk_api.pojos.enums.AppTaskStatusEnum;
import com.nokia.dingtalk_api.pojos.po.AppPo;
import com.nokia.dingtalk_api.pojos.po.AppTaskPo;
import com.nokia.dingtalk_api.pojos.po.ReceiveMessageLogPo;
import com.nokia.dingtalk_api.pojos.po.RobotCommandPo;
import com.nokia.dingtalk_api.pojos.po.RobotPo;
import com.nokia.dingtalk_api.pojos.po.SendMessageLogPo;
import com.nokia.dingtalk_api.pojos.vo.cms.GetAlertConfigVo;
import com.nokia.dingtalk_api.util.AesUtil;
import com.nokia.dingtalk_api.util.DingTalkApiUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;
import shade.com.alibaba.fastjson2.JSONObject;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 机器人接收消息
 * @link <a href="https://open.dingtalk.com/document/orgapp/robot-receive-message">...</a>
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class RobotCallbackService implements OpenDingTalkCallbackListener<JSONObject, JSONObject> {
    private final transient IRobotCommandService iRobotCommandService;
    private final transient OpenService openService;
    private final transient IRobotService iRobotService;
    private final transient AppRobotMapper appRobotMapper;
    private final transient IAppTaskService iAppTaskService;
    private final transient AppTaskService appTaskService;
    private final transient IReceiveMessageLogService iReceiveMessageLogService;
    private final transient ISendMessageLogService iSendMessageLogService;
    private final transient AlertConfigMapper alertConfigMapper;

    @Override
    public JSONObject execute(JSONObject request) {
        MDC.put("traceId", IdUtil.fastSimpleUUID());
        log.info("机器人接收消息: {}", request);
        // 消息类型
        String msgType = request.getString("msgtype");
        if (!"text".equals(msgType)) {
            return new JSONObject();
        }
        // 保存日志
        ReceiveMessageLogPo receiveMessageLogPo = new ReceiveMessageLogPo(request);
        String robotCode = receiveMessageLogPo.getRobotCode();
        RobotPo robotPo = iRobotService.getById(robotCode);
        receiveMessageLogPo.setRobotName(robotPo.getRobotName());
        iReceiveMessageLogService.save(receiveMessageLogPo);
        // 消息删除空白字符
        String content = StringUtils.trimAllWhitespace(receiveMessageLogPo.getContent());
        String[] split = content.split("#");
        // 查询机器人关联应用
        List<AppPo> robotApps = appRobotMapper.listRobotApp(robotCode);
        Map<String, AppPo> appMap = robotApps.stream().collect(Collectors.toMap(AppPo::getAppName, t -> t));
        // 返回用户id
        if ("1".equalsIgnoreCase(content)) {
            return getUserId(receiveMessageLogPo, robotPo);
        }
        // 返回群id
        if ("2".equals(receiveMessageLogPo.getConversationType()) && "2".equalsIgnoreCase(content)) {
            return getConversationId(receiveMessageLogPo, robotPo);
        }
        // 返回机器人关联的应用列表
        if ("3".equals(content)) {
            return getApp(robotApps, receiveMessageLogPo, robotPo);
        }
        // 返回应用操作命令
        if (StringUtils.startsWithIgnoreCase(content, "#查询命令")
                && split.length == 3 && appMap.containsKey(split[2])) {
            return getCommand(split, appMap, receiveMessageLogPo, robotPo);
        }
        // 执行命令调用应用方接口
        if (StringUtils.startsWithIgnoreCase(content, "#执行命令")
                && split.length > 3 && appMap.containsKey(split[2])) {
            return runCommand(split, appMap, robotPo, receiveMessageLogPo);
        }
        // 返回应用任务
        if (StringUtils.startsWithIgnoreCase(content, "#查询任务")
                && split.length == 3 && appMap.containsKey(split[2])) {
            return getTask(split, appMap, receiveMessageLogPo, robotPo);
        }
        // 执行应用任务
        if (StringUtils.startsWithIgnoreCase(content, "#执行任务")
                && split.length > 3 && appMap.containsKey(split[2])) {
            return runTask(split, appMap, receiveMessageLogPo, robotPo);
        }
        // 撤回消息
        if (StringUtils.startsWithIgnoreCase(content, "#撤回消息") && split.length == 3) {
            return recallMessages(split, receiveMessageLogPo, robotPo);
        }
        // 默认回复
        return defaultReply(receiveMessageLogPo, robotPo);
    }

    /**
     * 撤回消息
     */
    private JSONObject recallMessages(String[] split, ReceiveMessageLogPo receiveMessageLogPo, RobotPo robotPo) {
        try {
            String idString = split[2];
            Set<String> ids = Arrays.stream(idString.split(",")).collect(Collectors.toSet());
            GetAlertConfigVo alertConfig = alertConfigMapper.getConfig();
            List<SendMessageLogPo> sendMessageLogPos = iSendMessageLogService.listByIds(ids);
            SendMessageLogPo t1 = sendMessageLogPos.get(0);
            // 检查权限
            if (!robotPo.getRobotCode().equals(t1.getRobotCode())
                    && !robotPo.getRobotCode().equals(t1.getAlertRobotCode())
                    && (alertConfig == null || !robotPo.getRobotCode().equals(alertConfig.getRobotCode()))) {
                reply("没有权限", receiveMessageLogPo, robotPo);
                return new JSONObject();
            }
            String robotCode = robotPo.getRobotCode();
            String robotSecret = robotPo.getRobotSecret();
            if (!robotPo.getRobotCode().equals(t1.getRobotCode())) {
                RobotPo robot = iRobotService.getById(t1.getRobotCode());
                robotCode = robot.getRobotCode();
                robotSecret = robot.getRobotSecret();
            }
            // 撤回单聊消息
            if (t1.getConversationType().equals(1)) {
                String token = openService.getAccessToken(robotCode, robotSecret);
                BatchRecallOTOResponse r = DingTalkApiUtil.batchRecallOtoMessages(token, robotCode, ids.stream().toList());
                reply("撤回结果：" + r.body.toMap(), receiveMessageLogPo, robotPo);
                return new JSONObject();
            } else if (t1.getConversationType().equals(2)) {
                // 撤回群消息
                String token = openService.getAccessToken(robotCode, robotSecret);
                OrgGroupRecallResponse r = DingTalkApiUtil.recallGroupMessages(token, robotCode,
                        t1.getOpenConversationId(), ids.stream().toList());
                reply("撤回结果：" + r.body.toMap(), receiveMessageLogPo, robotPo);
                return new JSONObject();
            }
            return new JSONObject();
        } catch (Exception e) {
            Throwable rootCause = e;
            while (rootCause.getCause() != null && rootCause.getCause() != rootCause) {
                rootCause = rootCause.getCause();
            }
            reply(rootCause.getMessage(), receiveMessageLogPo, robotPo);
            return new JSONObject();
        }
    }

    /**
     * 默认回复
     */
    private JSONObject defaultReply(ReceiveMessageLogPo receiveMessageLogPo, RobotPo robotPo) {
        String s = """
帮助：
输入1 查询用户id
在群里@机器人并输入2 查询群id
输入3 查询应用列表
输入#查询命令#应用名称 查询该应用可用操作命令
输入#执行命令#应用名称#操作命令#参数（可选） 格式来执行命令
输入#查询任务#应用名称 查询该应用的任务列表
输入#执行任务#应用名称#任务名称#日期（可选，例如20240101） 执行应用任务
输入#撤回消息#消息id列表（英文逗号分隔） 撤回消息
""";
        reply(s, receiveMessageLogPo, robotPo);
        return new JSONObject();
    }

    /**
     * 执行应用任务
     */
    private JSONObject runTask(String[] split, Map<String, AppPo> appMap, ReceiveMessageLogPo receiveMessageLogPo,
                               RobotPo robotPo) {
        String appName = split[2];
        String taskName = split[3];
        String dataString = split.length > 4 ? split[4] : "";
        AppPo appPo = appMap.get(appName);
        QueryWrapper<AppTaskPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", appPo.getAppId());
        wrapper.eq("task_name", taskName);
        wrapper.ne("status", AppTaskStatusEnum.DELETED.value);
        AppTaskPo appTaskPo = iAppTaskService.getOne(wrapper);
        if (appTaskPo == null) {
            reply("任务不存在", receiveMessageLogPo, robotPo);
            return new JSONObject();
        }
        try {
            appTaskService.runAppTask(new RunAppTaskDto(appTaskPo.getTaskId(), dataString));
            reply("执行成功", receiveMessageLogPo, robotPo);
        } catch (Exception e) {
            reply("执行异常", receiveMessageLogPo, robotPo);
        }
        return new JSONObject();
    }

    /**
     * 返回应用任务
     */
    private JSONObject getTask(String[] split, Map<String, AppPo> appMap, ReceiveMessageLogPo receiveMessageLogPo,
                               RobotPo robotPo) {
        String appName = split[2];
        AppPo appPo = appMap.get(appName);
        QueryWrapper<AppTaskPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", appPo.getAppId());
        wrapper.ne("status", AppTaskStatusEnum.DELETED.value);
        wrapper.orderByAsc("task_id");
        List<AppTaskPo> list = iAppTaskService.list(wrapper);
        if (CollectionUtils.isEmpty(list)) {
            reply("没有查询到任务", receiveMessageLogPo, robotPo);
            return new JSONObject();
        }
        StringBuilder sb = new StringBuilder();
        sb.append(appPo.getAppName()).append("的任务：\n");
        int size = list.size();
        for (int i = 1; i <= size; i++) {
            AppTaskPo appTaskPo = list.get(i - 1);
            sb.append(i).append(".").append(appTaskPo.getTaskName()).append("\n");
        }
        reply(sb.toString(), receiveMessageLogPo, robotPo);
        return new JSONObject();
    }

    /**
     * 执行命令调用应用方接口
     */
    private JSONObject runCommand(String[] split, Map<String, AppPo> appMap, RobotPo robotPo,
                                  ReceiveMessageLogPo receiveMessageLogPo) {
        String appName = split[2];
        String operation = split[3];
        String parameter = split.length > 4 ? split[4] : "";
        AppPo appPo = appMap.get(appName);
        QueryWrapper<RobotCommandPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", appPo.getAppId());
        wrapper.eq("robot_code", robotPo.getRobotCode());
        wrapper.orderByAsc("command");
        List<RobotCommandPo> list = iRobotCommandService.list(wrapper);
        list.stream().filter(t -> t.getCommand().equals(operation)).findFirst().ifPresentOrElse(t -> {
            try {
                String token = AesUtil.generateToken(appPo.getAppId(), appPo.getAppSecret());
                Map<String, String> map = new HashMap<>();
                map.put("robotCode", robotPo.getRobotCode());
                map.put("senderStaffId", receiveMessageLogPo.getSenderStaffId());
                map.put("conversationType", receiveMessageLogPo.getConversationType());
                map.put("conversationId", receiveMessageLogPo.getConversationId());
                map.put("parameter", parameter);
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_JSON);
                headers.set("TOKEN", token);
                headers.set("APP_ID", appPo.getAppId());
                HttpEntity<Object> entity = new HttpEntity<>(map, headers);
                RestTemplate restTemplate = new RestTemplate();
                ResponseEntity<Object> response = restTemplate.postForEntity(t.getUrl(), entity, Object.class);
                log.info("调用应用方接口响应: {}", response);
                reply("执行成功", receiveMessageLogPo, robotPo);
            } catch (Exception e) {
                log.error(e.toString(), e);
                reply("执行异常", receiveMessageLogPo, robotPo);
            }
        }, () -> reply("找不到命令", receiveMessageLogPo, robotPo));
        return new JSONObject();
    }

    /**
     * 返回应用操作命令
     */
    private JSONObject getCommand(String[] split, Map<String, AppPo> appMap, ReceiveMessageLogPo receiveMessageLogPo,
                                  RobotPo robotPo) {
        String appName = split[2];
        AppPo appPo = appMap.get(appName);
        QueryWrapper<RobotCommandPo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", appPo.getAppId());
        wrapper.eq("robot_code", robotPo.getRobotCode());
        wrapper.orderByAsc("command");
        List<RobotCommandPo> list = iRobotCommandService.list(wrapper);
        if (CollectionUtils.isEmpty(list)) {
            reply("没有查询到操作命令", receiveMessageLogPo, robotPo);
            return new JSONObject();
        }
        StringBuilder sb = new StringBuilder();
        sb.append(appPo.getAppName()).append("的可用操作命令：\n");
        int size = list.size();
        for (int i = 1; i <= size; i++) {
            RobotCommandPo robotCommandPo = list.get(i - 1);
            sb.append(i).append(".").append(robotCommandPo.getCommand()).append("\n");
        }
        reply(sb.toString(), receiveMessageLogPo, robotPo);
        return new JSONObject();
    }

    /**
     * 返回机器人关联的应用列表
     */
    private JSONObject getApp(List<AppPo> robotApps, ReceiveMessageLogPo receiveMessageLogPo, RobotPo robotPo) {
        if (CollectionUtils.isEmpty(robotApps)) {
            reply("没有查询到应用", receiveMessageLogPo, robotPo);
            return new JSONObject();
        }
        StringBuilder sb = new StringBuilder();
        sb.append("应用列表：\n");
        int size = robotApps.size();
        for (int i = 1; i <= size; i++) {
            AppPo appPo = robotApps.get(i - 1);
            sb.append(i).append(".").append(appPo.getAppName()).append("\n");
        }
        reply(sb.toString(), receiveMessageLogPo, robotPo);
        return new JSONObject();
    }

    /**
     * 返回用户id
     */
    private JSONObject getUserId(ReceiveMessageLogPo receiveMessageLogPo, RobotPo robotPo) {
        reply("用户id: " + receiveMessageLogPo.getSenderStaffId(), receiveMessageLogPo, robotPo);
        return new JSONObject();
    }

    /**
     * 返回群id
     */
    private JSONObject getConversationId(ReceiveMessageLogPo receiveMessageLogPo, RobotPo robotPo) {
        reply("群id: " + receiveMessageLogPo.getConversationId(), receiveMessageLogPo, robotPo);
        return new JSONObject();
    }

    /**
     * 回复消息
     * @param content 内容
     * @param receiveMessageLogPo 收到的消息
     * @param robotPo 机器人信息
     */
    public void reply(String content, ReceiveMessageLogPo receiveMessageLogPo, RobotPo robotPo) {
        String accessToken = openService.getAccessToken(receiveMessageLogPo.getRobotCode(), robotPo.getRobotSecret());
        if ("2".equals(receiveMessageLogPo.getConversationType())) {
            // 发送群聊
            OrgGroupSendResponse r = DingTalkApiUtil.groupSendSampleText(content, accessToken,
                    receiveMessageLogPo.getConversationId(), receiveMessageLogPo.getRobotCode());
            iSendMessageLogService.save(new SendMessageLogPo(robotPo.getRobotCode(),
                    receiveMessageLogPo.getConversationId(), content, r));
        } else {
            // 发送单聊
            List<String> userIds = Collections.singletonList(receiveMessageLogPo.getSenderStaffId());
            BatchSendOTOResponse r = DingTalkApiUtil.batchSendOtoSampleText(content, accessToken,
                    receiveMessageLogPo.getRobotCode(), userIds);
            iSendMessageLogService.save(new SendMessageLogPo(robotPo.getRobotCode(), userIds, content, r));
        }
    }
}
