package com.nokia.dingtalk_api.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nokia.dingtalk_api.common.R;
import com.nokia.dingtalk_api.dao.IAppRobotService;
import com.nokia.dingtalk_api.dao.IRobotService;
import com.nokia.dingtalk_api.pojos.dto.cms.AddRobotDto;
import com.nokia.dingtalk_api.pojos.dto.cms.DeleteRobotDto;
import com.nokia.dingtalk_api.pojos.dto.cms.ListRobotDto;
import com.nokia.dingtalk_api.pojos.dto.cms.UpdateRobotDto;
import com.nokia.dingtalk_api.pojos.enums.SortEnum;
import com.nokia.dingtalk_api.pojos.po.AppRobotPo;
import com.nokia.dingtalk_api.pojos.po.RobotPo;
import com.nokia.dingtalk_api.pojos.vo.PageVo;
import com.nokia.dingtalk_api.pojos.vo.cms.GetRobotOptionsVo;
import com.nokia.dingtalk_api.pojos.vo.cms.ListRobotVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

@Service
@RequiredArgsConstructor
public class RobotService {
    private final IRobotService iRobotService;
    private final DingtalkClientService dingtalkClientService;
    private final AesService aesService;
    private final IAppRobotService iAppRobotService;

    public R<PageVo<ListRobotVo>> listRobot(ListRobotDto dto) {
        QueryWrapper<RobotPo> wrapper = new QueryWrapper<>();
        wrapper.eq("deleted", 0);
        if (StringUtils.hasText(dto.getRobotCode())) {
            wrapper.eq("robot_code", dto.getRobotCode());
        }
        if (StringUtils.hasText(dto.getRobotName())) {
            wrapper.like("robot_name", "%" + dto.getRobotName() + "%");
        }
        Page<RobotPo> page = new Page<>(dto.getCurrent(), dto.getPageSize());
        if (CollectionUtils.isEmpty(dto.getSort())) {
            page.addOrder(OrderItem.desc("update_time"));
        } else {
            List<OrderItem> items = dto.getSort().entrySet().stream()
                    .map(t -> t.getValue().equals(SortEnum.ascend) ? OrderItem.asc(t.getKey().value)
                            : OrderItem.desc(t.getKey().value)).toList();
            page.addOrder(items);
        }
        List<RobotPo> list = iRobotService.list(page, wrapper);
        List<ListRobotVo> vos = list.stream().map(ListRobotVo::new).toList();
        vos.forEach(vo -> vo.setRobotSecret(aesService.decrypt(vo.getRobotSecret())));
        return R.ok(new PageVo<>(page.getTotal(), vos));
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> addRobot(AddRobotDto dto) {
        RobotPo po = iRobotService.getById(dto.getRobotCode());
        if (po != null && po.getDeleted() == 0) {
            return R.error("机器人编号已存在");
        }
        dto.setRobotSecret(aesService.encrypt(dto.getRobotSecret()));
        po = new RobotPo(dto);
        iRobotService.saveOrUpdate(po);
        dingtalkClientService.update(po);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> updateRobot(UpdateRobotDto dto) {
        dto.setRobotSecret(aesService.encrypt(dto.getRobotSecret()));
        RobotPo po  = new RobotPo(dto);
        iRobotService.updateById(po);
        dingtalkClientService.update(po);
        return R.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    public R<Object> deleteRobot(DeleteRobotDto dto) {
        // 检查关联应用
        QueryWrapper<AppRobotPo> wrapper = new QueryWrapper<>();
        wrapper.eq("robot_code", dto.getRobotCode());
        boolean exists = iAppRobotService.exists(wrapper);
        if (exists) {
            return R.error("机器人被应用使用，无法删除");
        }
        RobotPo po  = new RobotPo(dto);
        iRobotService.updateById(po);
        dingtalkClientService.remove(dto.getRobotCode());
        return R.ok();
    }

    public R<List<GetRobotOptionsVo>> getRobotOptions() {
        List<RobotPo> list = iRobotService.list();
        List<GetRobotOptionsVo> vos = list.stream().map(GetRobotOptionsVo::new).toList();
        return R.ok(vos);
    }
}
