package com.nokia.dingtalk_api.util;

import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponse;
import com.aliyun.dingtalkrobot_1_0.Client;
import com.aliyun.dingtalkrobot_1_0.models.BatchRecallOTOHeaders;
import com.aliyun.dingtalkrobot_1_0.models.BatchRecallOTORequest;
import com.aliyun.dingtalkrobot_1_0.models.BatchRecallOTOResponse;
import com.aliyun.dingtalkrobot_1_0.models.BatchSendOTOHeaders;
import com.aliyun.dingtalkrobot_1_0.models.BatchSendOTORequest;
import com.aliyun.dingtalkrobot_1_0.models.BatchSendOTOResponse;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupRecallHeaders;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupRecallRequest;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupRecallResponse;
import com.aliyun.dingtalkrobot_1_0.models.OrgGroupSendResponse;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.models.RuntimeOptions;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiMediaUploadRequest;
import com.dingtalk.api.request.OapiV2UserGetbymobileRequest;
import com.dingtalk.api.response.OapiMediaUploadResponse;
import com.dingtalk.api.response.OapiV2UserGetbymobileResponse;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.nokia.dingtalk_api.common.exception.BizException;
import com.nokia.dingtalk_api.common.exception.MyRuntimeException;
import com.nokia.dingtalk_api.pojos.msg.SampleFile;
import com.nokia.dingtalk_api.pojos.msg.SampleImageMsg;
import com.nokia.dingtalk_api.pojos.msg.SampleMarkdown;
import com.nokia.dingtalk_api.pojos.msg.SampleText;
import com.taobao.api.ApiException;
import com.taobao.api.FileItem;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

/**
 * 机器人发送消息api工具
 */
@Slf4j
public class DingTalkApiUtil {
    /**
     * 机器人发送群聊文本类型消息
     * @param content 内容
     * @param token 调用服务端接口的授权凭证
     * @param openConversationId 会话ID
     * @param robotCode 机器人的编码
     */
    public static OrgGroupSendResponse groupSendSampleText(String content, String token, String openConversationId,
                                                           String robotCode) {
        SampleText t = SampleText.create(content);
        return groupSend(token, t.getMsgKey(), t.getMsgParam(), openConversationId, robotCode);
    }

    /**
     * 机器人发送群聊Markdown类型消息
     * @param title 标题
     * @param text 内容
     * @param token 调用服务端接口的授权凭证
     * @param openConversationId 会话ID
     * @param robotCode 机器人的编码
     */
    public static OrgGroupSendResponse groupSendSampleMarkdown(String title, String text, String token,
                                                               String openConversationId, String robotCode) {
        SampleMarkdown t = SampleMarkdown.create(title, text);
        return groupSend(token, t.getMsgKey(), t.getMsgParam(), openConversationId, robotCode);
    }

    /**
     * 机器人发送群聊图片类型消息
     * @param photoURL 图片
     * @param token 调用服务端接口的授权凭证
     * @param openConversationId 会话ID
     * @param robotCode 机器人的编码
     */
    public static OrgGroupSendResponse groupSendSampleImageMsg(String photoURL, String token, String openConversationId,
                                                               String robotCode) {
        SampleImageMsg t = SampleImageMsg.create(photoURL);
        return groupSend(token, t.getMsgKey(), t.getMsgParam(), openConversationId, robotCode);
    }

    /**
     * 机器人发送群聊文件类型消息
     * @param mediaId 通过上传媒体文件接口，获取media_id参数值
     * @param filename 文件名称
     * @param fileType 文件类型
     * @param token 调用服务端接口的授权凭证
     * @param openConversationId 会话ID
     * @param robotCode 机器人的编码
     */
    public static OrgGroupSendResponse groupSendSampleFile(String mediaId, String filename, String fileType,
                                                           String token, String openConversationId, String robotCode) {
        SampleFile t = SampleFile.create(mediaId, filename, fileType);
        return groupSend(token, t.getMsgKey(), t.getMsgParam(), openConversationId, robotCode);
    }

    /**
     * 机器人发送群聊消息
     * @link <a href="https://open.dingtalk.com/document/orgapp/the-robot-sends-a-group-message">...</a>
     * @param token 调用服务端接口的授权凭证
     * @param msgKey 消息模板key
     * @param msgParam 消息模板参数
     * @param openConversationId 会话ID
     * @param robotCode 机器人的编码
     */
    public static OrgGroupSendResponse groupSend(String token, String msgKey, String msgParam,
                                                 String openConversationId, String robotCode) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            com.aliyun.dingtalkrobot_1_0.Client client = new com.aliyun.dingtalkrobot_1_0.Client(getConfig());
            com.aliyun.dingtalkrobot_1_0.models.OrgGroupSendHeaders headers =
                    new com.aliyun.dingtalkrobot_1_0.models.OrgGroupSendHeaders();
            headers.xAcsDingtalkAccessToken = token;
            com.aliyun.dingtalkrobot_1_0.models.OrgGroupSendRequest request =
                    new com.aliyun.dingtalkrobot_1_0.models.OrgGroupSendRequest()
                            .setMsgParam(msgParam)
                            .setMsgKey(msgKey)
                            .setOpenConversationId(openConversationId)
                            .setRobotCode(robotCode);
            OrgGroupSendResponse response = client.orgGroupSendWithOptions(request, headers, new RuntimeOptions());
            log.info("OrgGroupSendResponse: {}", objectMapper.writeValueAsString(response));
            return response;
        } catch (TeaException e) {
            log.error(e.toString(), e);
            throw new BizException(e.getMessage());
        } catch (Exception e) {
            throw new MyRuntimeException(e);
        }
    }

    /**
     * 机器人批量发送文本类型消息
     * @param content 内容
     * @param token 调用服务端接口的授权凭证
     * @param robotCode 机器人的编码
     * @param userIds 接收机器人消息的用户的userId列表
     */
    public static BatchSendOTOResponse batchSendOtoSampleText(String content, String token, String robotCode,
                                                              List<String> userIds) {
        SampleText t = SampleText.create(content);
        return batchSendOto(token, t.getMsgKey(), t.getMsgParam(), robotCode, userIds);
    }

    /**
     * 机器人批量发送Markdown类型消息
     * @param title 标题
     * @param text 内容
     * @param token 调用服务端接口的授权凭证
     * @param robotCode 机器人的编码
     * @param userIds 接收机器人消息的用户的userId列表
     */
    public static BatchSendOTOResponse batchSendOtoSampleMarkdown(String title, String text, String token,
                                                                  String robotCode, List<String> userIds) {
        SampleMarkdown t = SampleMarkdown.create(title, text);
        return batchSendOto(token, t.getMsgKey(), t.getMsgParam(), robotCode, userIds);
    }

    /**
     * 机器人批量发送图片类型消息
     * @param photoURL 图片
     * @param token 调用服务端接口的授权凭证
     * @param robotCode 机器人的编码
     * @param userIds 接收机器人消息的用户的userId列表
     */
    public static BatchSendOTOResponse batchSendOtoSampleImageMsg(String photoURL, String token, String robotCode,
                                                                  List<String> userIds) {
        SampleImageMsg t = SampleImageMsg.create(photoURL);
        return batchSendOto(token, t.getMsgKey(), t.getMsgParam(), robotCode, userIds);
    }

    /**
     * 机器人发送群聊文件类型消息
     * @param mediaId 通过上传媒体文件接口，获取media_id参数值
     * @param filename 文件名称
     * @param fileType 文件类型
     * @param token 调用服务端接口的授权凭证
     * @param robotCode 机器人的编码
     * @param userIds 接收机器人消息的用户的userId列表
     */
    public static BatchSendOTOResponse batchSendOtoSampleFile(String mediaId, String filename, String fileType,
                                                              String token, String robotCode, List<String> userIds) {
        SampleFile t = SampleFile.create(mediaId, filename, fileType);
        return batchSendOto(token, t.getMsgKey(), t.getMsgParam(), robotCode, userIds);
    }

    /**
     * 批量发送人与机器人会话中机器人消息
     * @link <a href="https://open.dingtalk.com/document/orgapp/chatbots-send-one-on-one-chat-messages-in-batches">...</a>
     * @param token 调用服务端接口的授权凭证
     * @param msgKey 消息模板key
     * @param msgParam 消息模板参数
     * @param robotCode 机器人的编码
     * @param userIds 接收机器人消息的用户的userId列表
     */
    public static BatchSendOTOResponse batchSendOto(String token, String msgKey, String msgParam, String robotCode,
                                                    List<String> userIds) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            com.aliyun.dingtalkrobot_1_0.Client client = new com.aliyun.dingtalkrobot_1_0.Client(getConfig());
            BatchSendOTOHeaders headers = new BatchSendOTOHeaders();
            headers.xAcsDingtalkAccessToken = token;
            BatchSendOTORequest request = new BatchSendOTORequest()
                    .setRobotCode(robotCode)
                    .setUserIds(userIds)
                    .setMsgKey(msgKey)
                    .setMsgParam(msgParam);
            BatchSendOTOResponse response = client.batchSendOTOWithOptions(request, headers, new RuntimeOptions());
            log.info("BatchSendOTOResponse: {}", objectMapper.writeValueAsString(response));
            return response;
        } catch (TeaException e) {
            log.error(e.toString(), e);
            throw new BizException(e.getMessage());
        } catch (Exception e) {
            throw new MyRuntimeException(e);
        }
    }

    /**
     * 企业机器人撤回内部群消息
     * @link <a href="https://open.dingtalk.com/document/orgapp/enterprise-chatbot-withdraws-internal-group-messages">...</a>
     * @param token 调用服务端接口的授权凭证
     * @param robotCode 机器人的编码
     * @param openConversationId 会话ID
     * @param processQueryKeys 消息唯一标识列表
     */
    public static OrgGroupRecallResponse recallGroupMessages(String token, String robotCode, String openConversationId,
                                                                List<String> processQueryKeys) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            com.aliyun.dingtalkrobot_1_0.Client client = new Client(getConfig());
            OrgGroupRecallHeaders headers = new OrgGroupRecallHeaders();
            headers.xAcsDingtalkAccessToken = token;
            OrgGroupRecallRequest request = new OrgGroupRecallRequest()
                    .setOpenConversationId(openConversationId)
                    .setRobotCode(robotCode)
                    .setProcessQueryKeys(processQueryKeys);
            OrgGroupRecallResponse response = client.orgGroupRecallWithOptions(request, headers, new RuntimeOptions());
            log.info("OrgGroupRecallResponse: {}", objectMapper.writeValueAsString(response));
            return response;
        } catch (TeaException e) {
            log.error(e.toString(), e);
            throw new BizException(e.getMessage());
        } catch (Exception e) {
            throw new MyRuntimeException(e);
        }
    }

    /**
     * 批量撤回人与机器人会话中机器人消息
     * @link <a href="https://open.dingtalk.com/document/orgapp/batch-message-recall-chat">...</a>
     * @param token 调用服务端接口的授权凭证
     * @param robotCode 机器人的编码
     * @param processQueryKeys 消息唯一标识列表
     */
    public static BatchRecallOTOResponse batchRecallOtoMessages(String token, String robotCode,
                                                                List<String> processQueryKeys) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            com.aliyun.dingtalkrobot_1_0.Client client = new com.aliyun.dingtalkrobot_1_0.Client(getConfig());
            BatchRecallOTOHeaders headers = new BatchRecallOTOHeaders();
            headers.xAcsDingtalkAccessToken = token;
            BatchRecallOTORequest request = new BatchRecallOTORequest().setRobotCode(robotCode)
                    .setProcessQueryKeys(processQueryKeys);
            BatchRecallOTOResponse response = client.batchRecallOTOWithOptions(request, headers, new RuntimeOptions());
            log.info("BatchRecallOTOResponse: {}", objectMapper.writeValueAsString(response));
            return response;
        } catch (TeaException e) {
            log.error(e.toString(), e);
            throw new BizException(e.getMessage());
        } catch (Exception e) {
            throw new MyRuntimeException(e);
        }
    }

    /**
     * 上传媒体文件
     * @link <a href="https://open.dingtalk.com/document/orgapp/upload-media-files?spm=ding_open_doc.document.0.0.68fab17duL1H5G">...</a>
     * @param type 媒体文件类型：
     * image：图片，图片最大20MB。支持上传jpg、gif、png、bmp格式。
     * voice：语音，语音文件最大2MB。支持上传amr、mp3、wav格式。
     * video：视频，视频最大20MB。支持上传mp4格式。
     * file：普通文件，最大20MB。支持上传doc、docx、xls、xlsx、ppt、pptx、zip、pdf、rar格式。
     * @param media 要上传的媒体文件
     * @param token 调用该接口的应用凭证
     */
    public static OapiMediaUploadResponse upload(String type, FileItem media, String token) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/media/upload");
            OapiMediaUploadRequest request = new OapiMediaUploadRequest();
            request.setType(type);
            request.setMedia(media);
            OapiMediaUploadResponse response = client.execute(request, token);
            log.info("OapiMediaUploadResponse: {}", objectMapper.writeValueAsString(response));
            return response;
        } catch (ApiException e) {
            log.error(e.toString(), e);
            throw new BizException(e.getMessage());
        } catch (Exception e) {
            throw new MyRuntimeException(e);
        }
    }

    /**
     * 根据手机号查询用户
     * @link <a href="https://open.dingtalk.com/document/orgapp/query-users-by-phone-number">...</a>
     * @param token 调用服务端接口的授权凭证
     * @param mobile 用户的手机号
     */
    public static OapiV2UserGetbymobileResponse getUserByMobile(String token, String mobile) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/getbymobile");
        OapiV2UserGetbymobileRequest request = new OapiV2UserGetbymobileRequest();
        request.setMobile(mobile);
        OapiV2UserGetbymobileResponse response = client.execute(request, token);
        log.info("OapiV2UserGetbymobileResponse: {}", objectMapper.writeValueAsString(response));
        return response;
    }

    /**
     * 获取企业内部应用的accessToken
     * @link <a href="https://open.dingtalk.com/document/orgapp/obtain-the-access_token-of-an-internal-app">...</a>
     * @param appKey 已创建的企业内部应用的AppKey
     * @param appSecret 已创建的企业内部应用的AppSecret
     */
    public static GetAccessTokenResponse getAccessToken(String appKey, String appSecret) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            com.aliyun.dingtalkoauth2_1_0.Client client = new com.aliyun.dingtalkoauth2_1_0.Client(getConfig());
            com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenRequest request =
                    new com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenRequest()
                            .setAppKey(appKey)
                            .setAppSecret(appSecret);
            GetAccessTokenResponse response = client.getAccessToken(request);
            log.info("GetAccessTokenResponse: {}", objectMapper.writeValueAsString(response));
            return response;
        } catch (TeaException e) {
            log.error(e.toString(), e);
            throw new BizException(e.getMessage());
        } catch (Exception e) {
            throw new MyRuntimeException(e);
        }
    }

    public static com.aliyun.teaopenapi.models.Config getConfig() {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = "https";
        config.regionId = "central";
        return config;
    }
}
